@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * bftextview2_autocomp.c
  *
- * Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015 Olivier Sessink
+ * Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2026 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -195,6 +195,72 @@ static gboolean trigger_new_autocomp_idle(gpointer data) {
 	return FALSE;
 }
 
+static void 
+acwin_insert_current_selection(BluefishTextView * btv) {
+	GtkTreeSelection *selection;
+	GtkTreeIter it;
+	GtkTreeModel *model;
+	BluefishTextView *master = BLUEFISH_TEXT_VIEW(btv->master);
+	selection = gtk_tree_view_get_selection(ACWIN(btv->autocomp)->tree);
+	if (selection && gtk_tree_selection_get_selected(selection, &model, &it)) {
+		gchar *string;
+		gint stringlen;
+		gint existing_len, prefix_len;
+		guint pattern_id;
+		gint backup_chars = 0;
+		gboolean trigger_new_autocomp_popup=FALSE;
+		gtk_tree_model_get(model, &it, 1, &string, -1);
+		/*g_print("context %d has patternhash %p, string=%s\n",ACWIN(btv->autocomp)->contextnum, g_array_index(btv->bflang->st->contexts, Tcontext, ACWIN(btv->autocomp)->contextnum).patternhash, string); */
+		if (g_array_index
+			(master->bflang->st->contexts, Tcontext, ACWIN(btv->autocomp)->contextnum).patternhash) {
+			/*g_print("looking in context %d patternhash for '%s'\n",ACWIN(btv->autocomp)->contextnum, string); */
+			pattern_id =
+				GPOINTER_TO_INT(g_hash_table_lookup
+								(g_array_index
+								 (master->bflang->st->contexts, Tcontext,
+								  ACWIN(btv->autocomp)->contextnum).patternhash, string));
+			DBG_AUTOCOMP("got pattern_id=%d\n", pattern_id);
+			if (pattern_id) {
+				GSList *tmpslist =
+					g_array_index(master->bflang->st->matches, Tpattern, pattern_id).autocomp_items;
+				/* a pattern MAY have multiple autocomplete items. This code is not efficient if in the future some
+				   patterns would have many autocomplete items. I don't expect this, so I leave this as it is right now  */
+				while (tmpslist) {
+					Tpattern_autocomplete *pac = tmpslist->data;
+					if (g_strcmp0(string, pac->autocomplete_string) == 0) {
+						backup_chars = pac->autocomplete_backup_cursor;
+						trigger_new_autocomp_popup = (pac->trigger_new_autocomp_popup==1);
+					}
+					tmpslist = g_slist_next(tmpslist);
+				}
+			}
+		}
+		stringlen = strlen(string);
+		prefix_len = strlen(ACWIN(btv->autocomp)->prefix);
+		existing_len = get_existing_end_len(btv, string, prefix_len);
+
+		DBG_AUTOCOMP("acwin_check_keypress: ENTER: insert %s\n",
+					 string + prefix_len);
+		string[stringlen - existing_len] = '\0';
+		/* see if there are any \n characters in the autocompletion string, and add the current indenting to them */
+		string = string_maintain_indenting(btv, string, &backup_chars);
+		gtk_text_buffer_insert_at_cursor(btv->buffer, string + prefix_len, -1);
+		if (backup_chars != 0) {
+			GtkTextIter iter;
+			gtk_text_buffer_get_iter_at_mark(btv->buffer, &iter, gtk_text_buffer_get_insert(btv->buffer));
+			if (backup_chars-existing_len > 0 && gtk_text_iter_backward_chars(&iter, backup_chars-existing_len)) {
+				DBG_AUTOCOMP("move cursor %d chars back!\n", backup_chars);
+				gtk_text_buffer_place_cursor(btv->buffer, &iter);
+			}
+		}
+		if (trigger_new_autocomp_popup) {
+			g_idle_add(trigger_new_autocomp_idle, btv);
+		}
+		g_free(string);
+	}
+	acwin_cleanup(btv);
+}
+
 gboolean
 acwin_check_keypress(BluefishTextView * btv, GdkEventKey * event)
 {
@@ -205,68 +271,7 @@ acwin_check_keypress(BluefishTextView * btv, GdkEventKey * event)
 	}
 	switch (event->keyval) {
 	case GDK_Return:{
-			GtkTreeSelection *selection;
-			GtkTreeIter it;
-			GtkTreeModel *model;
-			BluefishTextView *master = BLUEFISH_TEXT_VIEW(btv->master);
-			selection = gtk_tree_view_get_selection(ACWIN(btv->autocomp)->tree);
-			if (selection && gtk_tree_selection_get_selected(selection, &model, &it)) {
-				gchar *string;
-				gint stringlen;
-				gint existing_len, prefix_len;
-				guint pattern_id;
-				gint backup_chars = 0;
-				gboolean trigger_new_autocomp_popup=FALSE;
-				gtk_tree_model_get(model, &it, 1, &string, -1);
-				/*g_print("context %d has patternhash %p, string=%s\n",ACWIN(btv->autocomp)->contextnum, g_array_index(btv->bflang->st->contexts, Tcontext, ACWIN(btv->autocomp)->contextnum).patternhash, string); */
-				if (g_array_index
-					(master->bflang->st->contexts, Tcontext, ACWIN(btv->autocomp)->contextnum).patternhash) {
-					/*g_print("looking in context %d patternhash for '%s'\n",ACWIN(btv->autocomp)->contextnum, string); */
-					pattern_id =
-						GPOINTER_TO_INT(g_hash_table_lookup
-										(g_array_index
-										 (master->bflang->st->contexts, Tcontext,
-										  ACWIN(btv->autocomp)->contextnum).patternhash, string));
-					DBG_AUTOCOMP("got pattern_id=%d\n", pattern_id);
-					if (pattern_id) {
-						GSList *tmpslist =
-							g_array_index(master->bflang->st->matches, Tpattern, pattern_id).autocomp_items;
-						/* a pattern MAY have multiple autocomplete items. This code is not efficient if in the future some
-						   patterns would have many autocomplete items. I don't expect this, so I leave this as it is right now  */
-						while (tmpslist) {
-							Tpattern_autocomplete *pac = tmpslist->data;
-							if (g_strcmp0(string, pac->autocomplete_string) == 0) {
-								backup_chars = pac->autocomplete_backup_cursor;
-								trigger_new_autocomp_popup = (pac->trigger_new_autocomp_popup==1);
-							}
-							tmpslist = g_slist_next(tmpslist);
-						}
-					}
-				}
-				stringlen = strlen(string);
-				prefix_len = strlen(ACWIN(btv->autocomp)->prefix);
-				existing_len = get_existing_end_len(btv, string, prefix_len);
-
-				DBG_AUTOCOMP("acwin_check_keypress: ENTER: insert %s\n",
-							 string + prefix_len);
-				string[stringlen - existing_len] = '\0';
-				/* see if there are any \n characters in the autocompletion string, and add the current indenting to them */
-				string = string_maintain_indenting(btv, string, &backup_chars);
-				gtk_text_buffer_insert_at_cursor(btv->buffer, string + prefix_len, -1);
-				if (backup_chars != 0) {
-					GtkTextIter iter;
-					gtk_text_buffer_get_iter_at_mark(btv->buffer, &iter, gtk_text_buffer_get_insert(btv->buffer));
-					if (backup_chars-existing_len > 0 && gtk_text_iter_backward_chars(&iter, backup_chars-existing_len)) {
-						DBG_AUTOCOMP("move cursor %d chars back!\n", backup_chars);
-						gtk_text_buffer_place_cursor(btv->buffer, &iter);
-					}
-				}
-				if (trigger_new_autocomp_popup) {
-					g_idle_add(trigger_new_autocomp_idle, btv);
-				}
-				g_free(string);
-			}
-			acwin_cleanup(btv);
+			acwin_insert_current_selection(btv);
 			return TRUE;
 		}
 		break;
@@ -366,6 +371,15 @@ acw_label_active_link_lcb(GtkLabel *label,gchar    *uri,gpointer  user_data) {
 	return TRUE;
 }
 
+static gboolean
+acw_clicked_lcb(GtkWidget *widget, GdkEventButton *event, gpointer acw) {
+	if(event->type == GDK_2BUTTON_PRESS) {
+		acwin_insert_current_selection(ACWIN(acw)->btv);
+		return TRUE;
+	}
+	return FALSE;
+}
+
 static Tacwin *
 acwin_create(BluefishTextView * btv)
 {
@@ -409,7 +423,7 @@ acwin_create(BluefishTextView * btv)
 	gtk_tree_view_set_enable_search(GTK_TREE_VIEW(acw->tree), FALSE);
 	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(acw->tree));
 	g_signal_connect(G_OBJECT(selection), "changed", G_CALLBACK(acw_selection_changed_lcb), acw);
-
+	g_signal_connect(G_OBJECT(acw->tree), "button-press-event", G_CALLBACK(acw_clicked_lcb), acw);
 /*	gtk_tree_view_set_search_column(GTK_TREE_VIEW(acw->tree),1);
 	gtk_tree_view_set_search_equal_func(GTK_TREE_VIEW(acw->tree),acwin_tree_search_lcb,prefix,NULL);*/
 
