@@ -2,7 +2,7 @@
  * Bluefish HTML Editor
  * bfwin.c
  *
- * Copyright (C) 2002-2013 Olivier Sessink
+ * Copyright (C) 2002-2026 Olivier Sessink
  * Copyright (C) 2011 James Hayward
  *
  * This program is free software; you can redistribute it and/or modify
@@ -51,6 +51,9 @@
 #include "bftextview2_spell.h"
 #endif /*HAVE_LIBENCHANT*/
 
+static void notebook_disconnect_signals(Tbfwin *bfwin);
+static void notebook_connect_signals(Tbfwin *bfwin);
+
 
 enum {
 	TARGET_URI_LIST,
@@ -106,6 +109,12 @@ notebooks_reorder(Tbfwin *bfwin, Tordering_mode mode)
 	/* now sort according to the order in the copylist */
 	for (tmplist=g_list_first(copylist);tmplist;tmplist=tmplist->next) {
 		DEBUG_MSG("notebooks_reorder, move document %s to position %d\n",gtk_label_get_text(GTK_LABEL(DOCUMENT(tmplist->data)->tab_menu)), i);
+#ifdef DUALNOTEBOOK
+		if (gtk_widget_get_parent(DOCUMENT(tmplist->data)->vsplit) == bfwin->notebook2) {
+			/* side-by-side, move to first notebook */
+			doc_move_notebook(bfwin->notebook2, bfwin->notebook, DOCUMENT(tmplist->data));
+		}
+#endif /* DUALNOTEBOOK */
 		gtk_notebook_reorder_child(GTK_NOTEBOOK(bfwin->notebook), DOCUMENT(tmplist->data)->vsplit,i);
 		i++;
 	}
@@ -116,24 +125,17 @@ static void
 notebook_move(Tbfwin * bfwin, gboolean move_left)
 {
 	gint curpos, newpos;
-
+	
 	if (!bfwin->current_document)
 		return;
-
-	curpos = gtk_notebook_page_num(GTK_NOTEBOOK(bfwin->notebook), bfwin->current_document->vsplit);
+	GtkWidget *notebook = gtk_widget_get_parent(bfwin->current_document->vsplit);
+	curpos = gtk_notebook_page_num(GTK_NOTEBOOK(notebook), bfwin->current_document->vsplit);
 	if (curpos != -1) {
-#ifdef DEVELOPMENT
-		{
-			GList *cur;
-			cur = g_list_nth(bfwin->documentlist, curpos);
-			g_return_if_fail(cur);
-		}
-#endif
 		newpos = curpos + ((move_left) ? -1 : 1);
 		DEBUG_MSG("notebook_move, cur=%d, new=%d (num_pages=%d)\n", curpos, newpos,
-				  gtk_notebook_get_n_pages(GTK_NOTEBOOK(bfwin->notebook)));
-		if (newpos >= 0 && newpos < gtk_notebook_get_n_pages(GTK_NOTEBOOK(bfwin->notebook)))
-			gtk_notebook_reorder_child(GTK_NOTEBOOK(bfwin->notebook), bfwin->current_document->vsplit,
+				  gtk_notebook_get_n_pages(GTK_NOTEBOOK(notebook)));
+		if (newpos >= 0 && newpos < gtk_notebook_get_n_pages(GTK_NOTEBOOK(notebook)))
+			gtk_notebook_reorder_child(GTK_NOTEBOOK(notebook), bfwin->current_document->vsplit,
 									   newpos);
 	}
 }
@@ -242,7 +244,7 @@ side_panel_notify_position(GObject * object, GParamSpec * pspec, gpointer data)
 	}
 	if (main_v->props.wrap_on_right_margin) {
 		/* all docs need to recalculate the amount of whitespace on the right */
-		g_print("side_panel_notify_position, recalc right margin\n");
+		DEBUG_MSG("side_panel_notify_position, recalc right margin\n");
 		bfwin_alldoc_recalc_right_margin(bfwin);
 	}
 }
@@ -462,8 +464,12 @@ bfwin_statusbar_show(Tbfwin * bfwin, gboolean active)
 }
 
 void
-bfwin_apply_session(Tbfwin * bfwin, Tdocument *active_doc)
+bfwin_apply_session(Tbfwin * bfwin, Tdocument *active_doc, Tuimode uimode)
 {
+	DEBUG_MSG("bfwin_apply_session, uimode=%d\n",uimode);
+	if (uimode != ui_undefined) {
+		default_ui_settings_apply(bfwin->session, uimode);
+	}
 	bfwin_set_main_toolbar_visible(bfwin, bfwin->session->view_main_toolbar, TRUE);
 
 	if (!bfwin_side_panel_show_hide_toggle(bfwin, FALSE, bfwin->session->view_left_panel, TRUE)) {
@@ -483,7 +489,7 @@ bfwin_apply_session(Tbfwin * bfwin, Tdocument *active_doc)
 		combobox_fill(bfwin->simplesearch_combo, NULL, bfwin->session->searchlist);
 	}
 	/* force this session in the plugins */
-	g_slist_foreach(main_v->plugins, bfplugins_enforce_session, bfwin);
+	plugins_enforce_session(bfwin, uimode);
 }
 
 void
@@ -575,7 +581,7 @@ bfwin_destroy_and_cleanup(Tbfwin *bfwin)
 {
 	GList *tmplist;
 	DEBUG_MSG("bfwin_destroy_and_cleanup, started for %p\n", bfwin);
-
+	notebook_disconnect_signals(bfwin);
 		/* all documents have to be freed for this window */
 	tmplist = g_list_first(bfwin->documentlist);
 	DEBUG_MSG("bfwin_cleanup, have %d documents in window %p\n", g_list_length(bfwin->documentlist),
@@ -588,10 +594,14 @@ bfwin_destroy_and_cleanup(Tbfwin *bfwin)
 		   left */
 		tmplist = g_list_first(bfwin->documentlist);
 	}
-
+	
 	g_signal_handler_disconnect(bfwin->notebook, bfwin->notebook_switch_signal);
+#ifdef DUALNOTEBOOK
+	g_signal_handler_disconnect(bfwin->notebook2, bfwin->notebook2_switch_signal);
+#endif
 	if (bfwin->update_searchhistory_idle_id != 0) {
 		g_source_remove(bfwin->update_searchhistory_idle_id);
+		bfwin->update_searchhistory_idle_id=0;
 	}
 
 	gtk_widget_destroy(bfwin->main_window);
@@ -615,6 +625,7 @@ bfwin_destroy_and_cleanup(Tbfwin *bfwin)
 
 	if (bfwin->notebook_changed_doc_activate_id != 0) {
 		g_source_remove(bfwin->notebook_changed_doc_activate_id);
+		bfwin->notebook_changed_doc_activate_id=0;
 	}
 
 	DEBUG_MSG("bfwin_cleanup, unref static actiongroups\n");
@@ -725,7 +736,7 @@ bfwin_osx_terminate_event(GtkWidget * widget, GdkEvent * event, Tbfwin * bfwin)
 
 	DEBUG_MSG("bfwin_osx_terminate_event, started for bfwin %p\n", bfwin);
 	if (!bfwin->documentlist) {
-		g_print("bfwin_osx_terminate_event, document list is empty\n");
+		DEBUG_MSG("bfwin_osx_terminate_event, document list is empty\n");
 		bfwin_destroy_and_cleanup(bfwin);
 		return TRUE;
 	}
@@ -1176,7 +1187,9 @@ notebook_switch_by_key(GtkWidget * widget, gpointer data)
 void
 notebook_unbind_tab_signals(Tbfwin * bfwin)
 {
+	DEBUG_MSG("notebook_unbind_tab_signals, started\n");
 	g_signal_handlers_disconnect_matched(bfwin->notebook, G_SIGNAL_MATCH_FUNC, 0, 0, NULL, notebook_switch_by_key, NULL);
+	DEBUG_MSG("notebook_unbind_tab_signals, finished\n");
 }
 
 void
@@ -1262,6 +1275,7 @@ notebook_set_tab_accels(Tbfwin * bfwin)
 #endif
 }
 
+#ifndef DUALNOTEBOOK
 static gboolean
 notebook_changed_activate_current_document(gpointer data)
 {
@@ -1274,20 +1288,31 @@ notebook_changed_activate_current_document(gpointer data)
 	DEBUG_MSG("notebook_changed_doc_activate_id=%d\n",BFWIN(data)->notebook_changed_doc_activate_id);
 	return FALSE;
 }
+#endif
 
 void
 bfwin_notebook_block_signals(Tbfwin * bfwin)
 {
+	DEBUG_MSG("bfwin_notebook_block_signals, started\n");
 	g_signal_handler_block(G_OBJECT(bfwin->notebook), bfwin->notebook_switch_signal);
+#ifdef DUALNOTEBOOK	
+	g_signal_handler_block(G_OBJECT(bfwin->notebook2), bfwin->notebook2_switch_signal);
+#endif
+	DEBUG_MSG("bfwin_notebook_block_signals, finished\n");
 }
 
 void
 bfwin_notebook_unblock_signals(Tbfwin * bfwin)
 {
+	DEBUG_MSG("bfwin_notebook_unblock_signals, started\n");
 	g_signal_handler_unblock(G_OBJECT(bfwin->notebook), bfwin->notebook_switch_signal);
+#ifdef DUALNOTEBOOK	
+	g_signal_handler_unblock(G_OBJECT(bfwin->notebook2), bfwin->notebook2_switch_signal);
+#endif
+	DEBUG_MSG("bfwin_notebook_unblock_signals, finished\n");
 }
 
-static void
+void
 bfwin_current_document_changed_notify(Tbfwin *bfwin, Tdocument *olddoc, Tdocument *newdoc) {
 	GSList *tmpslist;
 	for (tmpslist=bfwin->curdoc_changed;tmpslist;tmpslist=g_slist_next(tmpslist)) {
@@ -1343,9 +1368,140 @@ bfwin_document_destroy_remove_by_data(Tbfwin *bfwin, gpointer data)
 	callback_remove_by_data(&bfwin->doc_destroy, data);
 }
 
+/* 
+ * bfwin_notebook_changed(Tbfwin * bfwin, GtkWidget *notebook, gint newpage)
+ * 
+ * can be called with notebook==NULL and newpage==-1, in that case it is called to create a new document in an otherwise empty window
+ *
+this functions makes sure: 
+- that bfwin->current_document is set to the right pointer
+- that the current document is activated by an idle callback for bfwin_current_document_changed_notify()
+- that the syntax scanning for the previous current_document is disabled
+- that if the last document was closed and the window is not supposed to be closed, a new empty document is created
+- notifies plugins that the current document is changed by calling bfwin_current_document_changed_notify()
+
+after DUALNOTEBOOK the most important functions are:
+ - that the syntax scanning for non-visible documents is disabled
+ - that an empty document is created if there is no document anymore
+ - the visible documents are syntax scanned
+
+
+the function is called after notebook_switch_page signal
+after a keyboard-shortcut notebook cycle trough various tabs
+after a doc_destroy() 
+after a doc_close_multiple() --> if multiple documents are destroyed, we only call this function after all documents are gone
+after a project_load_from_file()
+*/
 void
-bfwin_notebook_changed(Tbfwin * bfwin, gint newpage)
+bfwin_notebook_changed(Tbfwin * bfwin, GtkWidget *notebook, gint newpage)
 {
+#ifdef DUALNOTEBOOK
+	/* the call to bfwin_current_document_changed_notify() is moved to document.c:doc_focus_in_lcb() 
+	setting the current_document is also moved to document.c:doc_focus_in_lcb()
+	
+	and calling the activate() from an idle callback is also moved to doc_focus_in_lcb()
+	*/
+	DEBUG_MSG("bfwin_notebook_changed, called with notebook %p and newpage %d\n",notebook,newpage);
+	
+	/* if we do have one last document, and we are called with a closing project, just close the project and return */
+	if (bfwin->current_document && bfwin->project && bfwin->project->close) {
+		project_final_close(bfwin, FALSE);
+		return;
+	}
+	gint notebooklen = gtk_notebook_get_n_pages(GTK_NOTEBOOK(bfwin->notebook));
+	guint doclistlen = g_list_length(bfwin->documentlist);
+	if (doclistlen==0) {
+		/* if there are no documents anymore in the window - create one */
+		bfwin->notebook_page = -1;
+		bfwin->notebook2_page = -1;
+		bfwin->notebook1_visible_document = NULL;
+		bfwin->notebook2_visible_document = NULL;
+		bfwin->current_document = doc_new(bfwin, TRUE);
+		bfwin_current_document_changed_notify(bfwin, NULL, bfwin->current_document);
+		gtk_widget_hide(bfwin->notebook2);
+		if (bfwin->project && bfwin->project->close) {
+			project_final_close(bfwin, FALSE);
+		}
+		return;
+	}
+	gint cur_notebook_page = gtk_notebook_get_current_page(GTK_NOTEBOOK(bfwin->notebook));
+	gint cur_notebook2_page = gtk_notebook_get_current_page(GTK_NOTEBOOK(bfwin->notebook2));
+	
+	/* if we have the situation that there are only documents in notebook2 left, and all documents in the normal notebook are gone, move all of them 
+	to the normal one and hide notebook2 */ 
+	if (cur_notebook_page == -1) {
+		notebook_disconnect_signals(bfwin);
+		GList *tmplist = g_list_first(bfwin->documentlist);
+		while (tmplist) {
+			doc_move_notebook(bfwin->notebook2, bfwin->notebook, DOCUMENT(tmplist->data));
+			tmplist = g_list_next(tmplist);
+		}
+		gtk_widget_hide(bfwin->notebook2);
+		bfwin->notebook_page = -1;
+		bfwin->notebook2_page = -1;
+		bfwin->notebook1_visible_document = NULL;
+		bfwin->notebook2_visible_document = NULL;
+		notebook_connect_signals(bfwin);
+		return;
+	}
+	
+	DEBUG_MSG("cur_notebook_page=%d bfwin->notebook_page=%d; cur_notebook2_page=%d bfwin->notebook2_page=%d\n",cur_notebook_page,bfwin->notebook_page,cur_notebook2_page,bfwin->notebook2_page);
+	Tdocument *tmpdoc;
+	if (cur_notebook_page != bfwin->notebook_page) {
+		/* de-activate scanning in old document, re-activate in newly visible document */
+		if (bfwin->notebook_page!=-1) {
+			tmpdoc = g_list_nth_data(bfwin->documentlist, bfwin->notebook_page);
+			if (tmpdoc) { /* if this document was destroyed we cannot find it anymore */
+				BLUEFISH_TEXT_VIEW(tmpdoc->view)->enable_scanner = FALSE;
+			}
+		}
+		if (cur_notebook_page!=-1) {
+			tmpdoc = g_list_nth_data(bfwin->documentlist, cur_notebook_page);
+			if (!tmpdoc) {
+				g_warning("error: no document found at position %d in the documentlist\n", cur_notebook_page);
+				return;
+			}
+			doc_becomes_visible(tmpdoc);
+			bfwin->notebook1_visible_document = tmpdoc;
+			if (notebook == bfwin->notebook) {
+				doc_set_current_schedule_activate(tmpdoc);
+				if (bfwin->current_document == NULL) {
+					bfwin->current_document = tmpdoc;
+				}
+			}
+		} else {
+			bfwin->notebook2_visible_document=NULL;
+		}
+		bfwin->notebook_page = cur_notebook_page;
+	}
+	if (cur_notebook2_page != bfwin->notebook2_page) {
+		/* de-activate scanning in old document, re-activate in newly visible document */
+		if (bfwin->notebook2_page!=-1) {
+			tmpdoc = g_list_nth_data(bfwin->documentlist, bfwin->notebook2_page + notebooklen);
+			if (tmpdoc) { /* if this document was destroyed we cannot find it anymore */
+				BLUEFISH_TEXT_VIEW(tmpdoc->view)->enable_scanner = FALSE;
+			}
+		}
+		if (cur_notebook2_page!=-1) {
+			tmpdoc = g_list_nth_data(bfwin->documentlist, cur_notebook2_page + notebooklen);
+			if (!tmpdoc) {
+				g_warning("error: no document found at position %d in the documentlist\n", cur_notebook2_page + notebooklen);
+				return;
+			}
+			doc_becomes_visible(tmpdoc);
+			bfwin->notebook2_visible_document = tmpdoc;
+			if (notebook == bfwin->notebook2) {
+				doc_set_current_schedule_activate(tmpdoc);
+			}
+		} else {
+			bfwin->notebook1_visible_document = NULL;
+			gtk_widget_hide(bfwin->notebook2);
+		}
+		bfwin->notebook2_page = cur_notebook2_page;
+	}
+	
+	
+#else
 	gint cur = newpage;
 	gint doclistlen;
 	Tdocument *prev_document;
@@ -1395,7 +1551,7 @@ bfwin_notebook_changed(Tbfwin * bfwin, gint newpage)
 		return;
 	}
 	if (bfwin->current_document) {
-		/*g_print("notebook_changed, set enable_scanner to FALSE for doc %p\n",bfwin->current_document); */
+		DEBUG_MSG("notebook_changed, set enable_scanner to FALSE for doc %p\n",bfwin->current_document); 
 		BLUEFISH_TEXT_VIEW(bfwin->current_document->view)->enable_scanner = FALSE;
 	}
 	prev_document = bfwin->current_document;
@@ -1405,7 +1561,7 @@ bfwin_notebook_changed(Tbfwin * bfwin, gint newpage)
 		return;
 	}
 	bfwin_current_document_changed_notify(bfwin, prev_document, bfwin->current_document);
-	
+
 	bfwin->last_notebook_page = cur;
 	DEBUG_MSG("bfwin_notebook_changed, current_document=%p, idle callback for doc activate = %d\n",
 			  bfwin->current_document, bfwin->notebook_changed_doc_activate_id);
@@ -1415,11 +1571,14 @@ bfwin_notebook_changed(Tbfwin * bfwin, gint newpage)
 			g_idle_add_full(NOTEBOOKCHANGED_DOCACTIVATE_PRIORITY, notebook_changed_activate_current_document,
 							bfwin, NULL);
 	}
+#endif /* DUALNOTEBOOK */
 }
 
-/*
+/* callback for the reordered signal on the notebook
 child : 	the child GtkWidget affected
 page_num : 	the new page number for child
+
+sets the document in the right location of the documentlist
 */
 static void
 notebook_reordered(GtkNotebook * notebook, GtkWidget * child, guint page_num, gpointer user_data)
@@ -1428,7 +1587,7 @@ notebook_reordered(GtkNotebook * notebook, GtkWidget * child, guint page_num, gp
 	Tdocument *doc = NULL;
 	GList *tmplist = g_list_first(bfwin->documentlist);
 
-	DEBUG_MSG("notebook_reordered, started\n");
+	DEBUG_MSG("notebook_reordered, started, notebook=%p, child=%p, page_num=%d\n",notebook,child,page_num);
 	/* look where this child (the GtkVPaned) is in the documentlist */
 	while (tmplist) {
 		if (DOCUMENT(tmplist->data)->vsplit == child) {
@@ -1440,10 +1599,75 @@ notebook_reordered(GtkNotebook * notebook, GtkWidget * child, guint page_num, gp
 	if (!doc)
 		return;
 	bfwin->documentlist = g_list_remove(bfwin->documentlist, doc);
+#ifdef DUALNOTEBOOK
+	if (GTK_WIDGET(notebook) == bfwin->notebook) {
+		DEBUG_MSG("notebook_reordered, moving doc %p to position %d in the documentlist\n", doc, page_num);
+		bfwin->documentlist = g_list_insert(bfwin->documentlist, doc, page_num);
+	} else if (GTK_WIDGET(notebook) == bfwin->notebook2) {
+		gint n1len = gtk_notebook_get_n_pages(GTK_NOTEBOOK(bfwin->notebook));
+		DEBUG_MSG("notebook_reordered, moving doc %p to position %d in the documentlist\n", doc, n1len+page_num);
+		bfwin->documentlist = g_list_insert(bfwin->documentlist, doc, n1len+page_num);
+	} else {
+		g_warning("notebook != bfwin->notebook && notebook != bfwin->notebook2 ??????\n");
+	}
+#else
 	DEBUG_MSG("notebook_reordered, moving doc %p to position %d in the documentlist\n", doc, page_num);
 	bfwin->documentlist = g_list_insert(bfwin->documentlist, doc, page_num);
+#endif
 	DEBUG_MSG("notebook_reordered, done\n");
 }
+#ifdef DUALNOTEBOOK
+
+gboolean bfwin_notebook_changed_idle_lcb(gpointer data) {
+	Tbfwin *bfwin = BFWIN(data);
+	bfwin_notebook_changed(bfwin,bfwin->notebook,-1);
+	return FALSE;
+}
+
+static void notebook_page_added(GtkNotebook *notebook,GtkWidget *child,guint page_num, gpointer user_data) {
+	Tbfwin *bfwin = BFWIN(user_data);
+	gint n1len=-1;
+	DEBUG_MSG("notebook_page_added, called notebook %p, child %p,page_num %d\n",notebook,child,page_num);
+	if (bfwin->notebook_removed_doc && DOCUMENT(bfwin->notebook_removed_doc)->vsplit == child) {
+		/* possibly the page that is added has bee removed from the other notebook, and we have to move it */
+		bfwin->documentlist = g_list_remove(bfwin->documentlist, bfwin->notebook_removed_doc);
+		if (GTK_WIDGET(notebook) == bfwin->notebook) {
+			DEBUG_MSG("notebook_reordered, moving doc %p to position %d in the documentlist\n", bfwin->notebook_removed_doc, page_num);
+			bfwin->documentlist = g_list_insert(bfwin->documentlist, bfwin->notebook_removed_doc, page_num);
+		} else if (GTK_WIDGET(notebook) == bfwin->notebook2) {
+			n1len = gtk_notebook_get_n_pages(GTK_NOTEBOOK(bfwin->notebook));
+			DEBUG_MSG("notebook_reordered, moving doc %p to position %d in the documentlist\n", bfwin->notebook_removed_doc, n1len+page_num);
+			bfwin->documentlist = g_list_insert(bfwin->documentlist, bfwin->notebook_removed_doc, n1len+page_num);
+		} else {
+			g_warning("notebook != bfwin->notebook && notebook != bfwin->notebook2 ??????\n");
+		}
+	}
+	bfwin->notebook_removed_doc = NULL;
+	if (n1len==0) {
+		DEBUG_MSG("notebook_page_added, notebook1 is empty, for call to bfwin_notebook_changed\n");
+		g_idle_add_full(NOTEBOOKCHANGED_DOCACTIVATE_PRIORITY, bfwin_notebook_changed_idle_lcb,bfwin, NULL);
+	}
+}
+
+static void notebook_page_removed(GtkNotebook *notebook,GtkWidget *child,guint page_num, gpointer user_data) {
+	Tbfwin *bfwin = BFWIN(user_data);
+	DEBUG_MSG("notebook_page_removed, called notebook %p, child %p,page_num %d\n",notebook,child,page_num);
+	/* 
+	if the document is destroyed, it is removed from the documentlist already. If it is going to be moved to another 
+	notebook it will be part of the documentlist, and we have to save it so we can add it in notebook_page_added()
+	*/
+	GList *tmplist = g_list_first(bfwin->documentlist);
+	while (tmplist) {
+		if (DOCUMENT(tmplist->data)->vsplit == child) {
+			bfwin->notebook_removed_doc = DOCUMENT(tmplist->data);
+			return;
+		}
+		tmplist = g_list_next(tmplist);
+	}
+}
+
+
+#endif
 
 /* Restores tab scrolling feature for gtk+3 builds */
 #if GTK_CHECK_VERSION (3, 0, 0)
@@ -1481,8 +1705,11 @@ notebook_scroll_event_lcb (GtkNotebook * notebook, GdkEventScroll *event, gpoint
 static void
 notebook_switch_page(GtkWidget * notebook, gpointer * page, gint page_num, Tbfwin * bfwin)
 {
+#ifdef DUALNOTEBOOK
+	DEBUG_MSG("notebook_switch_page, notebook %p, page %p, page_num %d\n",notebook,page,page_num);
+#endif
 	DEBUG_MSG("notebook_switch_page_lcb, page=%d\n", page_num);
-	bfwin_notebook_changed(bfwin, page_num);
+	bfwin_notebook_changed(bfwin, notebook, page_num);
 }
 
 static void
@@ -1494,6 +1721,27 @@ notebook_connect_signals(Tbfwin *bfwin)
 		g_signal_connect_after(G_OBJECT(bfwin->notebook), "switch-page", G_CALLBACK(notebook_switch_page),
 							   bfwin);
 	}
+#ifdef DUALNOTEBOOK	
+	DEBUG_MSG("notebook_connect_signals()\n");
+	if (bfwin->notebook2_switch_signal == 0) {
+		bfwin->notebook2_switch_signal =
+		g_signal_connect_after(G_OBJECT(bfwin->notebook2), "switch-page", G_CALLBACK(notebook_switch_page),
+							   bfwin);
+	}
+	if (bfwin->notebook_page_added_signal==0) {
+		bfwin->notebook_page_added_signal= g_signal_connect(G_OBJECT(bfwin->notebook), "page-added", G_CALLBACK(notebook_page_added), bfwin);
+	}
+	if (bfwin->notebook2_page_added_signal==0) {
+		bfwin->notebook2_page_added_signal= g_signal_connect(G_OBJECT(bfwin->notebook2), "page-added", G_CALLBACK(notebook_page_added), bfwin);
+	}
+	if (bfwin->notebook_page_removed_signal==0) {
+		bfwin->notebook_page_removed_signal= g_signal_connect(G_OBJECT(bfwin->notebook), "page-removed", G_CALLBACK(notebook_page_removed), bfwin);
+	}
+	if (bfwin->notebook2_page_removed_signal==0) {
+		bfwin->notebook2_page_removed_signal= g_signal_connect(G_OBJECT(bfwin->notebook2), "page-removed", G_CALLBACK(notebook_page_removed), bfwin);
+	}
+
+#endif
 }
 
 static void
@@ -1503,6 +1751,30 @@ notebook_disconnect_signals(Tbfwin *bfwin)
 		g_signal_handler_disconnect(bfwin->notebook, bfwin->notebook_switch_signal);
 		bfwin->notebook_switch_signal = 0;
 	}
+#ifdef DUALNOTEBOOK	
+	DEBUG_MSG("notebook_disconnect_signals()\n");
+	if (bfwin->notebook2_switch_signal > 0) {
+		g_signal_handler_disconnect(bfwin->notebook2, bfwin->notebook2_switch_signal);
+		bfwin->notebook2_switch_signal = 0;
+	}
+	if (bfwin->notebook_page_added_signal) {
+		g_signal_handler_disconnect(bfwin->notebook, bfwin->notebook_page_added_signal);
+		bfwin->notebook_page_added_signal=0;
+	}
+	if (bfwin->notebook2_page_added_signal) {
+		g_signal_handler_disconnect(bfwin->notebook2, bfwin->notebook2_page_added_signal);
+		bfwin->notebook2_page_added_signal=0;
+	}
+	if (bfwin->notebook_page_removed_signal) {
+		g_signal_handler_disconnect(bfwin->notebook, bfwin->notebook_page_removed_signal);
+		bfwin->notebook_page_removed_signal=0;
+	}
+	if (bfwin->notebook2_page_removed_signal) {
+		g_signal_handler_disconnect(bfwin->notebook2, bfwin->notebook2_page_removed_signal);
+		bfwin->notebook2_page_removed_signal=0;
+	}
+	
+#endif
 }
 
 static gboolean
@@ -1554,7 +1826,7 @@ bfwin_key_release_event(GtkWidget *widget, GdkEventKey  *kevent, gpointer   user
 			notebook_connect_signals(BFWIN(user_data));
 			i = gtk_notebook_get_current_page(GTK_NOTEBOOK(BFWIN(user_data)->notebook));
 			DEBUG_MSG("bfwin_key_release_event, connect signals, and call bfwin_notebook_changed(%p, %d)\n",user_data,i);
-			bfwin_notebook_changed(BFWIN(user_data), i);
+			bfwin_notebook_changed(BFWIN(user_data), widget, i);
 		}
 	}
 	return FALSE;
@@ -1565,7 +1837,10 @@ bfwin_create_main(Tbfwin * bfwin, gboolean with_new_doc)
 {
 	GtkWidget *vbox;
 	DEBUG_MSG("bfwin_create_main, bfwin=%p, main_window_w=%d\n", bfwin, main_v->globses.main_window_w);
-
+#ifdef DUALNOTEBOOK
+	bfwin->notebook_page=-1;
+	bfwin->notebook2_page=-1;
+#endif
 	bftextview2_identifier_hash_init(bfwin);
 	bfwin->main_window =
 		window_full2(_("New Bluefish Window"), 0, 0, G_CALLBACK(bfwin_destroy_event),
@@ -1649,7 +1924,30 @@ bfwin_create_main(Tbfwin * bfwin, gboolean with_new_doc)
 	/* Add notebook and its fake friend to their common hbox. */
 	bfwin->notebook_box = gtk_vbox_new(FALSE, 0);
 	gotoline_frame_create(bfwin);
+#ifdef DUALNOTEBOOK
+#if GTK_CHECK_VERSION(3,0,0)
+	bfwin->dualnotebooksplit = gtk_paned_new(GTK_ORIENTATION_HORIZONTAL);
+#else
+	bfwin->dualnotebooksplit = gtk_hpaned_new();
+#endif
+	gtk_paned_pack1(GTK_PANED(bfwin->dualnotebooksplit),bfwin->notebook,TRUE,FALSE);
+	gtk_box_pack_start(GTK_BOX(bfwin->notebook_box), bfwin->dualnotebooksplit, TRUE, TRUE, 0);
+	gtk_widget_show(bfwin->dualnotebooksplit);
+
+	bfwin->notebook2 = gtk_notebook_new();
+	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(bfwin->notebook2), main_v->props.document_tabposition);
+	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(bfwin->notebook2), TRUE);
+	gtk_notebook_set_show_border(GTK_NOTEBOOK(bfwin->notebook2), FALSE);
+	gtk_paned_pack2(GTK_PANED(bfwin->dualnotebooksplit),bfwin->notebook2,TRUE,FALSE);
+	gtk_widget_show(bfwin->notebook2);
+	
+	gtk_notebook_set_group_name(GTK_NOTEBOOK(bfwin->notebook),"bluefishdocuments");
+	gtk_notebook_set_group_name(GTK_NOTEBOOK(bfwin->notebook2),"bluefishdocuments");
+	gtk_notebook_set_scrollable(GTK_NOTEBOOK(bfwin->notebook2), TRUE);
+	/*gtk_widget_set_size_request(bfwin->notebook2, 150, 150);*/
+#else  /* DUALNOTEBOOK */
 	gtk_box_pack_start(GTK_BOX(bfwin->notebook_box), bfwin->notebook, TRUE, TRUE, 0);
+#endif /* DUALNOTEBOOK */
 	gtk_box_pack_start(GTK_BOX(bfwin->notebook_box), bfwin->notebook_fake, TRUE, TRUE, 0);
 
 	/* here we ask any plugins to build there gui, BEFORE WE START THE SIDEPANEL */
@@ -1694,6 +1992,10 @@ bfwin_create_main(Tbfwin * bfwin, gboolean with_new_doc)
 	}
 	notebook_connect_signals(bfwin);
 	g_signal_connect(G_OBJECT(bfwin->notebook), "page-reordered", G_CALLBACK(notebook_reordered), bfwin);
+#ifdef DUALNOTEBOOK
+	g_signal_connect(G_OBJECT(bfwin->notebook2), "page-reordered", G_CALLBACK(notebook_reordered), bfwin);
+#endif
+
 #if GTK_CHECK_VERSION (3, 0, 0) /* Restores tab scrolling feature for gtk+3 builds */
 	gtk_widget_add_events (bfwin->notebook, GDK_SCROLL_MASK);
 	g_signal_connect (G_OBJECT(bfwin->notebook), "scroll-event", G_CALLBACK (notebook_scroll_event_lcb), bfwin);
@@ -2018,3 +2320,84 @@ bfwin_has_document_by_pointer(Tbfwin * bfwin, Tdocument * document)
 	return TRUE;
 }
 
+
+void default_ui_settings_apply(Tsessionvars * sessionvars, gint uimode) {
+	DEBUG_MSG("default_ui_settings_apply, uimode=%d\n",uimode);
+	sessionvars->view_left_panel = (uimode != 5);
+	sessionvars->view_main_toolbar = (uimode != 4 && uimode != 5);
+	sessionvars->view_statusbar = (uimode != 4 && uimode != 5);
+}
+
+static void default_ui_radio_toggled(GtkWidget *widget, gpointer data) {
+	Tdefault_ui *du = (Tdefault_ui *)data;
+	DEBUG_MSG("default_uit_radio_toggled, widget=%p, du=%p\n",widget,du);
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))) {
+		for (int i=0;i<5;i++) {
+			if (widget == du->radio[i]) {
+				du->uimode=i+1;
+				break;
+			}
+		}
+		gchar *imagefilenames[] = {"ui_everything.png", "ui_webdev.png", "ui_dev.png", "ui_experienced.png", "ui_minimal.png", NULL};
+		gchar *modetext[] = {
+			N_("<i>Every toolbar, menubar, statusbar and sidebar enabled by default.</i>"),
+			N_("<i>Default user interface with the HTML toolbar enabled.</i>"),
+			N_("<i>Default user interface with menu, main toolbar, sidebar and statusbar.</i>"),
+			N_("<i>No toolbars, just menu, sidebar and statusbar.</i>"),
+			N_("<i>Nothing except the menu.</i>"),
+			NULL
+		};
+		if (du->uimode > 0 && du->uimode <= 5 && imagefilenames[du->uimode-1]) {
+			gchar *filepath = g_strconcat(PKGDATADIR"/", imagefilenames[du->uimode-1], NULL);
+			DEBUG_MSG("ui_mode=%d, load file %s\n",du->uimode,filepath);
+			gtk_image_set_from_file(GTK_IMAGE(du->image), filepath);
+			gtk_label_set_markup(GTK_LABEL(du->subtext), gettext(modetext[du->uimode-1]));
+			g_free(filepath);
+		}
+	}
+}
+
+/**
+ * default_ui_settings
+ * returns a GtkContainer that can be added to the project dialog, or used for the 
+ * very first Bluefish run by the about plugin
+ *
+ */
+
+Tdefault_ui *default_ui_settings(const gchar *labeltext, Tdefault_ui *du) {
+	GtkWidget *label;
+
+	du->grid = compat_gridtable_new(6,2,TRUE);
+	gtk_container_set_border_width(GTK_CONTAINER(du->grid),3);
+	compat_gridtable_set_column_spacing(GTK_WIDGET(du->grid),20);
+	compat_gridtable_set_row_spacing(GTK_WIDGET(du->grid),8);
+	
+	label = gtk_label_new(NULL);
+	gtk_label_set_markup(GTK_LABEL(label), labeltext);
+	compat_gridtable_attach(GTK_WIDGET(du->grid),label,0,0,2,1);
+	
+	du->radio[0] = gtk_radio_button_new_with_label(NULL, _("Everything enabled"));
+	compat_gridtable_attach(GTK_WIDGET(du->grid),du->radio[0],0,1,1,1);
+	g_signal_connect(du->radio[0], "toggled", G_CALLBACK(default_ui_radio_toggled), du);
+	du->radio[1] = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(du->radio[0]), _("Web development default"));
+	compat_gridtable_attach(GTK_WIDGET(du->grid),du->radio[1],0,2,1,1);
+	g_signal_connect(du->radio[1], "toggled", G_CALLBACK(default_ui_radio_toggled), du);
+	du->radio[2] = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(du->radio[1]), _("Programming default"));
+	compat_gridtable_attach(GTK_WIDGET(du->grid),du->radio[2],0,3,1,1);
+	g_signal_connect(du->radio[2], "toggled", G_CALLBACK(default_ui_radio_toggled), du);
+	du->radio[3] = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(du->radio[2]), _("Experienced users default"));
+	compat_gridtable_attach(GTK_WIDGET(du->grid),du->radio[3],0,4,1,1);
+	g_signal_connect(du->radio[3], "toggled", G_CALLBACK(default_ui_radio_toggled), du);
+	du->radio[4] = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(du->radio[3]), _("Minimalistic"));
+	compat_gridtable_attach(GTK_WIDGET(du->grid),du->radio[4],0,5,1,1);
+	g_signal_connect(du->radio[4], "toggled", G_CALLBACK(default_ui_radio_toggled), du);
+	
+	du->image = gtk_image_new(); /*gtk_image_new_from_file(PKGDATADIR"/ui_everything.png");*/
+	compat_gridtable_attach(GTK_WIDGET(du->grid),du->image,1,1,1,4);
+	du->subtext = gtk_label_new(NULL);
+	compat_gridtable_attach(GTK_WIDGET(du->grid),du->subtext,1,5,1,1);
+	
+	default_ui_radio_toggled(du->radio[0], du);
+	return du;
+}
+
