@@ -2,7 +2,7 @@
  * Bluefish HTML Editor
  * bfwin.h
  *
- * Copyright (C) 2002-2011 Olivier Sessink
+ * Copyright (C) 2002-2026 Olivier Sessink
  * Copyright (C) 2011 James Hayward
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,6 +24,23 @@
 
 #include "bluefish.h"
 
+typedef enum {
+	ui_undefined = 0,
+	ui_everything = 1,
+	ui_webdevfull = 2,
+	ui_traditional = 3,
+	ui_lean = 4,
+	ui_minimal = 5
+} Tuimode;
+
+typedef struct {
+	GtkWidget *grid;
+	GtkWidget *image;
+	GtkWidget *subtext;
+	GtkWidget *radio[5];
+	Tuimode uimode;
+} Tdefault_ui;
+
 typedef void (*DocDestroyCallback) (Tdocument *doc, gpointer data);
 typedef void (*CurdocChangedCallback) (Tbfwin *bfwin, Tdocument *olddoc, Tdocument *newdoc, gpointer data);
 typedef void (*DocDeleteRangeCallback) (Tdocument *doc, GtkTextIter * itstart, gint start, GtkTextIter * itend, gint end, const gchar *string, gpointer data);
@@ -41,6 +58,8 @@ void bfwin_simplesearch_from_clipboard(Tbfwin *bfwin);
 void notebook_set_tab_accels(Tbfwin * bfwin);
 void notebook_bind_tab_signals(Tbfwin * bfwin);
 void notebook_unbind_tab_signals(Tbfwin * bfwin);
+
+void bfwin_current_document_changed_notify(Tbfwin *bfwin, Tdocument *olddoc, Tdocument *newdoc);
 void bfwin_notebook_block_signals(Tbfwin * bfwin);
 void bfwin_notebook_unblock_signals(Tbfwin * bfwin);
 
@@ -54,7 +73,8 @@ void bfwin_document_delete_range_remove_by_data(Tbfwin *bfwin, gpointer data);
 void bfwin_document_destroy_register(Tbfwin *bfwin, DocDestroyCallback func, gpointer data);
 void bfwin_document_destroy_remove_by_data(Tbfwin *bfwin, gpointer data);
 
-void bfwin_notebook_changed(Tbfwin * bfwin, gint newpage);
+void bfwin_notebook_changed(Tbfwin * bfwin, GtkWidget *notebook, gint newpage);
+gboolean bfwin_notebook_changed_idle_lcb(gpointer data);
 void bfwin_notebook_switch(Tbfwin * bfwin, guint action);
 
 void bfwin_main_toolbar_show(Tbfwin * bfwin, gboolean active);
@@ -68,7 +88,7 @@ void bfwin_side_panel_rebuild(Tbfwin * bfwin);
 
 void bfwin_statusbar_show(Tbfwin * bfwin, gboolean active);
 
-void bfwin_apply_session(Tbfwin * bfwin, Tdocument *active_doc);
+void bfwin_apply_session(Tbfwin * bfwin, Tdocument *active_doc, Tuimode uimode);
 void bfwin_apply_settings(Tbfwin * bfwin);
 void bfwin_destroy_and_cleanup(Tbfwin *bfwin);
 gboolean bfwin_delete_event(GtkWidget * widget, GdkEvent * event, Tbfwin * bfwin);
@@ -93,4 +113,9 @@ void bfwin_set_title(Tbfwin * bfwin, Tdocument * doc, gint num_modified_change);
 gboolean bfwin_switch_to_document_by_index(Tbfwin * bfwin, gint index);
 gboolean bfwin_switch_to_document_by_pointer(Tbfwin * bfwin, Tdocument * document);
 gboolean bfwin_has_document_by_pointer(Tbfwin * bfwin, Tdocument * document);
+
+
+void default_ui_settings_apply(Tsessionvars * sessionvars, gint uimode);
+Tdefault_ui *default_ui_settings(const gchar *label, Tdefault_ui *du);
+
 #endif							/* BFWIN_H_ */
