@@ -3,7 +3,7 @@
  * bfwin_uimanager.c
  *
  * Copyright (C) 2011, 2015, 2017 James Hayward and Olivier Sessink
- * Copyright (C) 2017 - 2025 Olivier Sessink
+ * Copyright (C) 2017 - 2026 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -85,7 +85,13 @@ ui_floating_window(GtkAction * action, gpointer user_data)
 {
 	doc_floating_view_new(BFWIN(user_data));
 }
-
+#ifdef DUALNOTEBOOK
+static void 
+ui_doc_sidebyside(GtkAction * action, gpointer user_data)
+{
+	doc_sidebyside(BFWIN(user_data));
+}
+#endif /* DUALNOTEBOOK */
 static void
 ui_font_size_decrease(GtkAction * action, gpointer user_data)
 {
@@ -454,6 +460,14 @@ ui_file_save_copy(GtkAction * action, gpointer user_data)
 		doc_save_backend(bfwin->current_document, docsave_copy, FALSE, FALSE);
 }
 
+static void
+ui_file_save_as_template(GtkAction * action, gpointer user_data)
+{
+	if (BFWIN(user_data)->current_document) {
+		file_save_as_template(BFWIN(user_data));
+	}
+}
+
 static void
 ui_file_save(GtkAction * action, gpointer user_data)
 {
@@ -529,12 +543,15 @@ ui_open_from_selection(GtkAction * action, gpointer user_data)
 {
 	file_open_from_selection(BFWIN(user_data));
 }
-
+#ifndef WIN32
+#ifndef MAC_INTEGRATION
 static void
 ui_upload_download_dialog(GtkAction * action, gpointer user_data)
 {
 	sync_dialog(BFWIN(user_data));
 }
+#endif
+#endif
 
 static void
 ui_quit(GtkAction * action, gpointer user_data)
@@ -894,6 +911,10 @@ static const GtkActionEntry global_actions[] = {
 	 G_CALLBACK(ui_encoding_add_remove)},
 	{"DocFloatingWindow", NULL, N_("Floati_ng Window"), NULL, N_("Open current document in floating window"),
 	 G_CALLBACK(ui_floating_window)},
+#ifdef DUALNOTEBOOK
+	{"DocSideBySide", NULL, N_("Side by side (experimental)"), NULL, N_("Document side-by-side (experimental)"),
+	 G_CALLBACK(ui_doc_sidebyside)},
+#endif /* DUALNOTEBOOK */
 	{"DocFontSizeIncrease", NULL, N_("_Increase"), "<control>equal", N_("Increase document font size"),
 	 G_CALLBACK(ui_font_size_increase)},
 	{"DocFontSizeDecrease", NULL, N_("_Decrease"), "<control>minus", N_("Decrease document font size"),
@@ -1000,6 +1021,8 @@ static const GtkActionEntry document_actions[] = {
 	 G_CALLBACK(ui_file_reload)},
 	{"FileSaveCopy", NULL, N_("Save a cop_y..."), "<shift><control>y", N_("Save a copy"),
 	 G_CALLBACK(ui_file_save_copy)},
+	{"FileSaveAsTemplate", NULL, N_("Save as Template..."), "", N_("Save contents as template"),
+	 G_CALLBACK(ui_file_save_as_template)},
 	{"FileInsert", NULL, N_("_Insert..."), NULL, N_("Insert file"), G_CALLBACK(ui_insert_doc)},	/* Required to move here to disable sensitivity on OSX */
 	{"FileRename", NULL, N_("Rena_me..."), "F2", N_("Rename file"), G_CALLBACK(ui_file_rename)},
 #ifdef MAC_INTEGRATION			/* On Mac we configure this to alt and then moving them to control */
