@@ -452,7 +452,7 @@ int main(int argc, char *argv[])
 		g_free(cwd);
 	}
 #endif
-	rcfile_check_directory();
+	/*rcfile_check_directory();*/
 	rcfile_parse_main();
 #ifdef ENABLE_NLS
 	if (main_v->props.language!=NULL && main_v->props.language[0]!='\0') {
