@@ -2,7 +2,7 @@
  * bluefish.h - global prototypes
  *
  * Copyright (C) 1998 Olivier Sessink and Chris Mazuc
- * Copyright (C) 1999-2023 Olivier Sessink
+ * Copyright (C) 1999-2026 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,6 +32,7 @@
 /*#define DEBUG_PATHS*/
 
 #define ENABLEPLUGINS
+#define DUALNOTEBOOK
 
 #include "config.h"
 
@@ -65,7 +66,11 @@
 #endif
 
 #ifdef DEBUG
+#ifdef WIN32
+#define DEBUG_MSG g_message
+#else
 #define DEBUG_MSG g_print
+#endif
 #define DEBUG_MSG_C g_critical
 #define DEBUG_MSG_E g_error
 #define DEBUG_MSG_W g_warning
@@ -258,7 +263,7 @@ typedef struct {
 	gboolean load_first; /*sets priority of async loading */
 	gushort close_doc;
 	gushort close_window;
-
+	gint last_tabsize_pixels;
 	GList *need_autosave;		/* if autosave is needed, a direct pointer to main_v->need_autosave; */
 	GList *autosave_progress;
 	gpointer autosave_action;
@@ -316,6 +321,7 @@ typedef struct {
 	gint cursor_size;
 	gint highlight_cursor;
 	gint save_accelmap; 	/* save the accelerator map on exit */
+	gint dynamic_menu_shortcut; /* 0 means disabled, 1 means enabled */
 	gint max_recent_files;		/* length of Open Recent list */
 	gint max_dir_history;		/* length of directory history */
 	gint backup_file;			/* wheather to use a backup file */
@@ -372,6 +378,7 @@ typedef struct {
 	gint delay_scan_time;
 	gint autocomp_popup_mode;	/* delayed or immediately */
 	gint autocomp_min_prefix_len; /* minimum number of matching characters before autocomp is activated */
+	gchar *tagedit_accel_string;
 	gboolean reduced_scan_triggers;
 	gint autosave;
 	gint autosave_time;
@@ -550,10 +557,25 @@ typedef struct {
 	guint filebrowser_merge_id;
 
 	GtkWidget *menubar;
-	gint last_notebook_page;	/* a check to see if the notebook changed to a new page */
 	guint notebook_changed_doc_activate_id;
-	guint statusbar_pop_id;
 	guint notebook_switch_signal;
+#ifdef DUALNOTEBOOK
+	guint notebook2_switch_signal;
+	guint notebook_page_added_signal;
+	guint notebook_page_removed_signal;
+	guint notebook2_page_added_signal;
+	guint notebook2_page_removed_signal;
+	
+	guint notebook_split_page;
+	gint notebook_page;	/* last known page for notebook  */
+	gint notebook2_page;	/* last known page for notebook2, -1 if no page known or no pages available in notebook2 */
+	Tdocument *notebook1_visible_document;
+	Tdocument *notebook2_visible_document;
+	gpointer notebook_removed_doc; /* removed from one notebook, but probably soon added to the other notebook */
+#else
+	gint last_notebook_page;	/* a check to see if the notebook changed to a new page */
+#endif
+	guint statusbar_pop_id;
 	guint update_searchhistory_idle_id;
 	GtkWidget *gotoline_entry;
 	GtkWidget *simplesearch_combo;
@@ -565,6 +587,10 @@ typedef struct {
 	GtkWidget *notebook;
 	GtkWidget *notebook_fake;
 	GtkWidget *notebook_box;	/* Container for notebook and notebook_fake */
+#ifdef DUALNOTEBOOK
+	GtkWidget *notebook2;
+	GtkWidget *dualnotebooksplit;
+#endif /* DUALNOTEBOOK */
 	GtkWidget *middlebox;		/* holds the document notebook, OR the hpaned with the left panel AND the document notebook */
 	GtkWidget *vpane;			/* holds the middlebox AND the outputbox (which might be NULL) */
 	GtkWidget *hpane;			/* we need this to show/hide the filebrowser */
@@ -651,6 +677,8 @@ typedef struct {
 											word the user clicked */
 	guint autocomp_accel_key;				 /* by default <ctrl><space> activates autocompletion */
 	GdkModifierType autocomp_accel_mods; /* but this shortcut is also used to switch input languages for example by chinese users */
+	guint tagedit_accel_key;				 /* by default <ctrl><t> activates tag edit */
+	GdkModifierType tagedit_accel_mods; 
 	gpointer bmarkdata;
 	gint num_untitled_documents;
 	gchar *securedir;			/* temporary rwx------ directory for secure file creation */
