@@ -2,7 +2,7 @@
  * document.h - global function for document handling
  *
  * Copyright (C) 1998 Olivier Sessink and Chris Mazuc
- * Copyright (C) 1999-2015 Olivier Sessink
+ * Copyright (C) 1999-2026 Olivier Sessink
  * Copyright (C) 2011 James Hayward
  *
  * This program is free software; you can redistribute it and/or modify
@@ -52,12 +52,13 @@ void alldocs_foreach(foreachdocfunc func, gpointer data);
 gint have_modified_documents(GList * doclist);
 GList *return_urilist_from_doclist(GList * doclist);
 GList *return_arraylist_from_doclist(GList * doclist);
+gint doc_offset_from_line(Tdocument *doc, gint line);
 void add_filename_to_recentlist(Tbfwin * bfwin, GFile * uri);
 void remove_filename_from_recentlist(Tbfwin * bfwin, gboolean project, GFile * uri);
 
 gint documentlist_return_index_from_uri(GList * doclist, GFile * uri);
 void doc_set_uri(Tdocument * doc, GFile * uri, gboolean on_destroy, gboolean copy_bookmarks);
-Tdocument *documentlist_return_document_from_uri(GList * doclist, GFile * uri);
+Tdocument *documentlist_return_document_from_uri(GFile * uri);
 Tdocument *documentlist_return_document_from_index(GList * doclist, gint index);
 gint document_return_num_notcomplete(GList * doclist);
 
@@ -117,6 +118,9 @@ void doc_destroy(Tdocument * doc, gboolean delay_activation);
 void document_unset_filename(Tdocument * doc);
 gint doc_save(Tdocument * doc, gboolean do_save_as, gboolean do_move, gboolean window_closing);
 void document_set_show_symbols(Tdocument * doc, gboolean value);
+#ifdef DUALNOTEBOOK
+void doc_set_current_schedule_activate(Tdocument *doc);
+#endif
 void doc_recalculate_right_margin(Tdocument *doc);
 void bfwin_alldoc_recalc_right_margin(Tbfwin *bfwin);
 Tdocument *doc_new(Tbfwin * bfwin, gboolean delay_activate);
@@ -129,6 +133,9 @@ void doc_new_from_input(Tbfwin * bfwin, gchar * input, gboolean delay_activate,
 void docs_new_from_files(Tbfwin * bfwin, GList * file_list, gboolean move_to_this_win);
 void doc_reload(Tdocument * doc, GFileInfo * newfinfo, gboolean warn_user);
 void doc_start_modified_check(Tdocument * doc);
+#ifdef DUALNOTEBOOK
+void doc_becomes_visible(Tdocument *doc);
+#endif
 void doc_activate(Tdocument * doc);
 void doc_force_activate(Tdocument * doc);
 
@@ -154,6 +161,10 @@ GList *list_relative_document_filenames(Tdocument * curdoc);
 void doc_jump(Tdocument * doc);
 
 void doc_floating_view_new(Tbfwin * bfwin);
+#ifdef DUALNOTEBOOK
+void doc_move_notebook(GtkWidget *srcnotebook, GtkWidget *dstnotebook, Tdocument *doc);
+void doc_sidebyside(Tbfwin *bfwin);
+#endif /* DUALNOTEBOOK */
 void doc_split_view(Tdocument * doc, gboolean enable);
 
 #endif							/* __DOCUMENT_H_ */
