@@ -81,6 +81,7 @@ during startup:
 
 #include "bluefish.h"
 #include "bf_lib.h"
+#include "paths_state_config.h"
 #include "bfwin.h"
 #include "document.h"
 #include "file.h"
@@ -117,7 +118,8 @@ create_autosave_path(Tdocument * doc, GHashTable * excludehash)
 		main_v->props.autosave_location_mode = 0;
 	}
 	if (main_v->props.autosave_location_mode == 0 || doc->uri == NULL) {
-		gchar *path, *dir = g_strconcat(g_get_home_dir(), "/." PACKAGE "/autosave/", NULL);
+		gchar *path, *dir;
+		dir = paths_autosavedir_char();
 		if (doc->uri) {
 			gchar *base = g_file_get_basename(doc->uri);
 			path = unique_path(dir, base, excludehash);
@@ -148,7 +150,9 @@ static void
 autosave_save_journal(void)
 {
 	GFile *file;
-	gchar *path = g_strdup_printf("%s/." PACKAGE "/autosave_journal.%d", g_get_home_dir(), getpid());
+	gchar *journaldir = paths_autosavejournaldir_char();
+	gchar *path = g_strdup_printf("%s/autosave_journal.%d", journaldir, getpid());
+	g_free(journaldir);
 	DEBUG_MSG("autosave_save_journal in %s\n", path);
 	file = g_file_new_for_path(path);
 	g_free(path);
@@ -379,7 +383,7 @@ autosave_cleanup_old_autosaves(gpointer data)
 	gchar *dir;
 	const gchar *tmp;
 	DEBUG_MSG("autosave_cleanup_old_autosaves\n");
-	dir = g_strconcat(g_get_home_dir(), "/." PACKAGE "/autosave/", NULL);
+	dir = paths_autosavedir_char();
 	gdir = g_dir_open(dir, 0, &error);
 	if (error) {
 		g_warning("failed to open directory %s for cleanup\n", dir);
@@ -399,79 +403,109 @@ autosave_cleanup_old_autosaves(gpointer data)
 	return FALSE;
 }
 
+static gint autosave_search_journal(Tbfwin *bfwin, const gchar *journaldir) {
+	gboolean found_journal=FALSE;
+	const gchar *tmp;
+	GPatternSpec *patspec;
+	GDir *gdir;
+	GError *gerror = NULL;
+	
+	gdir = g_dir_open(journaldir, 0, &gerror);
+	if (gerror) {
+		g_warning("failed to open %s to recover autosaved files with error %s\n", journaldir, gerror->message);
+		g_error_free(gerror);
+		return -2;
+	}
+
+	patspec = g_pattern_spec_new("autosave_journal.*");
+	tmp = g_dir_read_name(gdir);
+	while (tmp) {
+		int tmplen = strlen(tmp);
+		if (tmp && tmp[0] != '\0' && tmp[tmplen - 1] != '~'
+			&& g_pattern_match(patspec, tmplen, tmp, NULL)) {
+			gint pid;
+			found_journal = TRUE;
+			pid = get_int_from_string(tmp);
+#ifdef WIN32
+			HANDLE hProc;
+			if (pid > 0 && !(hProc = OpenProcess(PROCESS_QUERY_INFORMATION, FALSE, pid)))
+#else
+			if (pid > 0 && kill(pid, 0) != 0) /* process pid is not alive, or not our process which also means we should recover */
+#endif
+				{
+				gchar *path;
+				GError *gerror = NULL;
+				GFile *file;
+				path = g_strconcat(journaldir, tmp, NULL);
+				file = g_file_new_for_path(path);
+				autosave_recover(bfwin, file);
+				g_file_delete(file, NULL, &gerror);
+				if (gerror) {
+					g_warning("failed to delete recovered autosave_journal %s: %s\n", path,
+							  gerror->message);
+				}
+				g_free(path);
+				g_object_unref(file);
+			}
+#ifdef WIN32
+			else
+				CloseHandle(hProc);
+#endif
+		}
+		tmp = g_dir_read_name(gdir);
+	}
+	
+	g_dir_close(gdir);
+	g_pattern_spec_free(patspec);
+
+	return found_journal;
+}
+
 void
 autosave_init(gboolean recover, Tbfwin * bfwin)
 {
 	DEBUG_MSG("autosave_init, recover=%d\n", recover);
 	if (main_v->autosave_id) {
 		g_source_remove(main_v->autosave_id);
+		main_v->autosave_id=0;
 	}
 	main_v->autosave_need_journal_save = FALSE;
 
 	if (recover && bfwin) {
-		GError *error = NULL;
-		const gchar *tmp;
-		GPatternSpec *patspec;
-		GDir *gdir;
 		gboolean found_journal = FALSE;
-		gchar *dir;
+		gboolean possible_xdg_migration_needed = FALSE;
+		gchar *dir, *journaldir;
 
 		/* test if autosave directory exists */
-		dir = g_strconcat(g_get_home_dir(), "/." PACKAGE "/autosave/", NULL);
+		dir = paths_autosavedir_char();
+		journaldir = paths_autosavejournaldir_char();
+		
 		if (!g_file_test(dir, G_FILE_TEST_EXISTS)) {
-			g_mkdir(dir, 0700);
-		}
-		g_free(dir);
-
-		dir = g_strconcat(g_get_home_dir(), "/." PACKAGE "/", NULL);
-		gdir = g_dir_open(dir, 0, &error);
-
-		if (error) {
-			g_warning("failed to open %s to recover autosaved files\n", dir);
-			g_free(dir);
-			g_error_free(error);
-			return;
-		}
-
-		patspec = g_pattern_spec_new("autosave_journal.*");
-		tmp = g_dir_read_name(gdir);
-		while (tmp) {
-			int tmplen = strlen(tmp);
-			if (tmp && tmp[0] != '\0' && tmp[tmplen - 1] != '~'
-				&& g_pattern_match(patspec, tmplen, tmp, NULL)) {
-				gint pid;
-				found_journal = TRUE;
-				pid = get_int_from_string(tmp);
-#ifdef WIN32
-				HANDLE hProc;
-				if (pid > 0 && !(hProc = OpenProcess(PROCESS_QUERY_INFORMATION, FALSE, pid))) {
-#else
-				if (pid > 0 && kill(pid, 0) != 0) {	/* process pid is not alive, or not our process which also means we should recover */
-#endif
-					gchar *path;
-					GError *error = NULL;
-					GFile *file;
-					path = g_strconcat(dir, tmp, NULL);
-					file = g_file_new_for_path(path);
-					autosave_recover(bfwin, file);
-					g_file_delete(file, NULL, &error);
-					if (error) {
-						g_warning("failed to delete recovered autosave_journal %s: %s\n", path,
-								  error->message);
-					}
-					g_free(path);
-					g_object_unref(file);
-				}
-#ifdef WIN32
-				else
-					CloseHandle(hProc);
-#endif
+			/* we might have to migrate autosave files from a bluefish 2.2.19 or 
+			older to the XDG-base-dir-spec file locations, APPDATA or 
+			Library/Aapplication Support /. 
+			
+			see if we have to restore from ~/.bluefish/ */
+			gchar *dirold = g_strconcat(g_get_home_dir(), "/." PACKAGE "/autosave/", NULL);
+			if (g_file_test(dirold, G_FILE_TEST_EXISTS)) {
+				possible_xdg_migration_needed = TRUE;
 			}
-			tmp = g_dir_read_name(gdir);
+			g_print("autosave_init, create %s\n",dir);
+			g_mkdir_with_parents(dir, 0700);
 		}
 		g_free(dir);
-		g_dir_close(gdir);
-		g_pattern_spec_free(patspec);
+
+		
+		found_journal = autosave_search_journal(bfwin, journaldir);
+		if (possible_xdg_migration_needed && found_journal == 0) {
+			/* do the migration from 2.2.19 or older that did not use the XDG BASE DIR spec for file locations */
+			g_free(journaldir);
+			journaldir = g_strconcat(g_get_home_dir(), "/." PACKAGE "/", NULL);
+			found_journal = autosave_search_journal(bfwin, journaldir);
+		} 
+		
+		g_free(journaldir);
+
 		if (!found_journal) {
 			/* if there was no journal we can remove *ALL* autosave files */
 			g_idle_add_full(G_PRIORITY_LOW, autosave_cleanup_old_autosaves, NULL, NULL);
@@ -489,7 +523,9 @@ autosave_cleanup(void)
 {
 	GFile *file;
 	GError *error = NULL;
-	gchar *path = g_strdup_printf("%s/." PACKAGE "/autosave_journal.%d", g_get_home_dir(), getpid());
+	gchar *journaldir = paths_autosavejournaldir_char();
+	gchar *path = g_strdup_printf("%s/autosave_journal.%d", journaldir, getpid());
+	g_free(journaldir);
 	DEBUG_MSG("autosave_cleanup\n");
 	file = g_file_new_for_path(path);
 
