@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * file_dialogs.c - file dialogs
  *
- * Copyright (C) 2005-2018 Olivier Sessink
+ * Copyright (C) 2005-2026 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -38,6 +38,8 @@
 #include "snr3.h"				/* snr3_run_extern_replace() */
 #include "stringlist.h"
 #include "undo_redo.h"
+#include "paths_state_config.h"
+#include "bfwin_uimanager.h"
 
 static gchar *modified_on_disk_warning_string(const gchar * filename, GFileInfo * oldfinfo,
 											  GFileInfo * newfinfo);
@@ -317,17 +319,21 @@ file_open_cb(GtkWidget * widget, Tbfwin * bfwin)
   gtk_widget_destroy(dialog);*/
 }
 
+/* functions to save template borrow the same structure and callbacks from open URL */
 typedef struct {
 	Tbfwin *bfwin;
 	GtkWidget *win;
 	GtkWidget *entry;
-} Tou;
+} Tou; 
+
+/* functions to save template borrow this same callback */
 static void
 open_url_destroy_lcb(GtkWidget * widget, Tou * ou)
 {
 	g_free(ou);
 }
 
+/* functions to save template borrow this same callback */
 static void
 open_url_cancel_lcb(GtkWidget * widget, Tou * ou)
 {
@@ -450,8 +456,7 @@ doc_checkNsave_lcb(TcheckNsave_status status, GError * gerror, gpointer data)
 			gint retval;
 			gchar *tmpstr =
 				g_strdup_printf(_
-								("A backupfile for %s could not be created. If you continue, this file will be overwritten."),
-gtk_label_get_text(GTK_LABEL(doc->tab_label)));
+								("A backupfile for %s could not be created. If you continue, this file will be overwritten."), gtk_label_get_text(GTK_LABEL(doc->tab_label)));
 			retval =
 				message_dialog_new_multi(BFWIN(doc->bfwin)->main_window, GTK_MESSAGE_WARNING, buttons,
 										 _("File backup failure"), tmpstr);
@@ -1090,6 +1095,101 @@ doc_save_all_close(Tbfwin * bfwin)
 	g_list_free(duplist);
 }
 
+static TcheckNsave_return 
+checknsave_template_lcb(TcheckNsave_status status, GError * gerror, gpointer data) {
+	switch (status) {
+		case CHECKANDSAVE_ERROR:
+		case CHECKANDSAVE_ERROR_NOBACKUP:
+		case CHECKANDSAVE_ERROR_NOWRITE:
+		case CHECKANDSAVE_ERROR_MODIFIED:
+		case CHECKANDSAVE_ERROR_CANCELLED:
+			g_warning("failed to save as template, error: %s",gerror?gerror->message:"unknown");
+		break;
+		case CHECKANDSAVE_FINISHED:
+			main_v->templates = g_list_prepend(main_v->templates, (gchar **)data);
+			GList *tmplist = g_list_first(main_v->bfwinlist);
+			while (tmplist) {
+				bfwin_templates_menu_create(BFWIN(tmplist->data));
+				tmplist = g_list_next(tmplist);
+			}
+		break;
+	}
+	return CHECKNSAVE_STOP;
+}
+
+static void
+save_as_template_ok_lcb(GtkWidget * widget, Tou * ou)
+{
+	gchar *name = gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(ou->entry));
+	gchar **arr2store;
+	Trefcpointer *buffer;
+	gpointer savep;
+	DEBUG_MSG("save_as_template_ok_lcb, name=%s\n", name);
+	
+	/* now actually save the document as template */
+	GFile *uri = paths_templatesdir_gfile();
+	GFile *child = g_file_get_child(uri, name);
+
+	arr2store = array_from_arglist(name,g_file_get_path(child), NULL); 
+	buffer = refcpointer_new(doc_get_chars(ou->bfwin->current_document, 0, -1));
+	savep = file_checkNsave_uri_async(child, NULL, buffer, strlen(buffer->data), FALSE ,FALSE, checknsave_template_lcb, arr2store, ou->bfwin);
+	
+	g_object_unref(uri);
+	g_object_unref(child);
+	refcpointer_unref(buffer);
+	gtk_widget_destroy(ou->win);
+}
+/* functions to save template borrow the same callbacks from open URL */
+
+/**
+ * file_save_as_template:
+  * @bfwin: #Tbfwin* bfwin pointer
+ *
+ * opens a dialog where you can enter the template name
+ *
+ * Return value: void
+ **/
+void
+file_save_as_template(Tbfwin * bfwin)
+{
+	GtkWidget *align, *vbox, *hbox, *but;
+	GList *tmplist, *newlist=NULL;
+	Tou *ou = g_new(Tou, 1);
+	ou->bfwin = bfwin;
+	ou->win =
+		window_full2(_("Save as template"), GTK_WIN_POS_CENTER_ON_PARENT, 12, G_CALLBACK(open_url_destroy_lcb), ou,
+					 TRUE, bfwin->main_window);
+	gtk_widget_set_size_request(ou->win, 450, -1);
+	vbox = gtk_vbox_new(FALSE, 5);
+	gtk_container_add(GTK_CONTAINER(ou->win), vbox);
+	gchar **arr;	
+	for (tmplist=g_list_first(main_v->templates);tmplist;tmplist=tmplist->next) {
+		arr = tmplist->data;
+		newlist = g_list_prepend(newlist, arr[0]);
+	}
+	ou->entry = boxed_combobox_with_popdown("", newlist, TRUE, vbox);
+	g_list_free(newlist);
+	align = gtk_alignment_new(0.0, 1.0, 1.0, 0.0);
+	gtk_alignment_set_padding(GTK_ALIGNMENT(align), 12, 0 ,0, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), align, FALSE, FALSE, 0);
+
+#if GTK_CHECK_VERSION(3,0,0)
+	hbox = gtk_button_box_new(GTK_ORIENTATION_HORIZONTAL);
+#else
+	hbox = gtk_hbutton_box_new();
+#endif
+	gtk_button_box_set_layout(GTK_BUTTON_BOX(hbox), GTK_BUTTONBOX_END);
+	gtk_box_set_spacing(GTK_BOX(hbox), 6);
+	gtk_container_add(GTK_CONTAINER(align), hbox);
+	but = bf_stock_cancel_button(G_CALLBACK(open_url_cancel_lcb), ou);
+	gtk_box_pack_start(GTK_BOX(hbox), but, FALSE, TRUE, 0);
+	but = bf_stock_ok_button(G_CALLBACK(save_as_template_ok_lcb), ou);
+	gtk_box_pack_start(GTK_BOX(hbox), but, FALSE, TRUE, 0);
+	gtk_window_set_default(GTK_WINDOW(ou->win), but);
+	DEBUG_MSG("file_save_as_template, showing win\n");
+	gtk_widget_show_all(ou->win);
+}
+
 /**
  * file_close_cb:
  * @widget: unused #GtkWidget
@@ -1140,8 +1240,10 @@ doc_close_multiple_backend(Tbfwin * bfwin, gboolean close_window, Tclose_mode cl
 	}
 	g_list_free(duplist);
 	DEBUG_MSG("doc_close_multiple_backend, finished\n");
-	if (!close_window)
-		bfwin_notebook_changed(bfwin, -1);
+	if (!close_window) {
+		DEBUG_MSG("doc_close_multiple_backend, finished, call bfwin_notebook_changed so there will be a new document created\n");
+		bfwin_notebook_changed(bfwin, bfwin->notebook, -1);
+	}
 }
 
 void
