@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * file_treemodel.c - non-GUI general functions
  *
- * Copyright (C) 2013 Olivier Sessink
+ * Copyright (C) 2013-2017 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -448,7 +448,8 @@ gchar *gfile_display_name(GFile * uri, GFileInfo * finfo)
 static void fill_uri(UriRecord * newrecord, GFile * uri, GFileInfo * finfo)
 {
 	GIcon *icon;
-	newrecord->name = g_strdup(g_file_info_get_display_name(finfo));
+	/* as pango string we want the g_file_info_get_display_name(), but in the store we want g_file_info_get_name() */
+	newrecord->name = g_strdup(g_file_info_get_name(finfo));
 	newrecord->fast_content_type = g_strdup(g_file_info_get_attribute_string(finfo, G_FILE_ATTRIBUTE_STANDARD_FAST_CONTENT_TYPE));
 	/*newrecord->name_collate_key = g_utf8_collate_key(newrecord->name, -1); *//* for fast sorting, used later */
 	newrecord->uri = uri;
@@ -798,6 +799,12 @@ static void add_multiple_uris(Turi_in_refresh *uir, GList * finfolist)
 		GFileInfo *finfo = tmplist->data;
 		/* don't allocate new memory for 'name', if it already exists we don't need it */
 		newrecord->name = (gchar *) g_file_info_get_name(finfo);
+#ifdef DEVELOPMENT
+		if (g_strcmp0(g_file_info_get_name(finfo),g_file_info_get_display_name(finfo))!=0) {
+			g_print("name=%s\n",g_file_info_get_name(finfo));
+			g_print("displayname=%s\n",g_file_info_get_display_name(finfo));
+		}
+#endif
 		newrecord->isdir = (g_file_info_get_file_type(finfo) == G_FILE_TYPE_DIRECTORY);
 		DEBUG_MSG("bsearch for name %s\n", newrecord->name);
 		tmp = bsearch(&newrecord, *rows, old_num_rows, sizeof(UriRecord *), compare_records);
@@ -814,7 +821,7 @@ static void add_multiple_uris(Turi_in_refresh *uir, GList * finfolist)
 			DEBUG_MSG("%s does not yet exist\n", newrecord->name);
 			/* this file does not exist */
 			uir->dir_changed = TRUE;
-			child = g_file_get_child(precord->uri, newrecord->name);
+			child = g_file_get_child(precord->uri, g_file_info_get_name(finfo));
 			fill_uri(newrecord, child, finfo);
 			g_object_unref(child);
 #ifdef DEVELOPMENT
@@ -1184,6 +1191,7 @@ void filetreemodel_add_file(FileTreemodel * ftm, GFile * uri, const gchar *conte
 	bname = g_file_get_basename(uri);
 	if (!content_type) {
 		guesstype = g_content_type_guess(bname, NULL, 0, NULL);
+		/*g_print("guesstype=%s for bname=%s\n",guesstype,bname);*/
 	}
 	if (content_type || guesstype) {
 		GIcon *icon;
@@ -1191,7 +1199,7 @@ void filetreemodel_add_file(FileTreemodel * ftm, GFile * uri, const gchar *conte
 		icon_name = icon_name_from_icon(icon);
 		g_object_unref(icon);
 	}
-	add_single_uri(ftm, precord, uri, bname, content_type, icon_name, FALSE, weight);
+	add_single_uri(ftm, precord, uri, bname, content_type?content_type:guesstype, icon_name, FALSE, weight);
 	g_free(bname);
 	g_free(icon_name);
 	g_object_unref(puri);
