@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * gtk_easy.c
  *
- * Copyright (C) 1999-2025 Olivier Sessink
+ * Copyright (C) 1999-2026 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -1274,7 +1274,8 @@ file_chooser_custom_filter_func(GtkFileFilterInfo * filter_info, gpointer data)
 		return TRUE;
 	if (!cf->filter->filetypes)
 		return !cf->filter->mode;*/
-	DEBUG_MSG("file_chooser_custom_filter_func, calling file_visible_in_filter for %s with mime=%s and filename=%s\n", filter_info->display_name, filter_info->mime_type, filter_info->filename);
+	if (filter_info)
+		DEBUG_MSG("file_chooser_custom_filter_func, calling file_visible_in_filter for %s with mime=%s and filename=%s\n", filter_info->display_name, filter_info->mime_type, filter_info->filename);
 	/* for some reason the GtkFileFilterInfo often has a NULL filename, so we use the display name to filter on */
 	return file_visible_in_filter(cf->filter, filter_info->mime_type, filter_info->display_name);
 	/*ret = GPOINTER_TO_INT(g_hash_table_lookup(cf->filter->filetypes, filter_info->mime_type));
@@ -1609,7 +1610,6 @@ color_but_new2(GtkWidget * entry)
 }
 
 #if GTK_CHECK_VERSION(3,0,0)
-#ifndef MAC_INTEGRATION
 /* ********************************************************************
  * following code implements dynamic menu shortcut setting that was
  * possible in gtk2 but was removed in gtk3 
@@ -1620,11 +1620,13 @@ static GtkWidget *hovered_item = NULL;
 /* Hover tracking */
 static gboolean on_item_enter(GtkWidget *item, GdkEventCrossing *e, gpointer d) {
 	hovered_item = item;
+	/*g_print("on_item_enter, hovered_item=%p\n",hovered_item);*/
 	return FALSE;
 }
 
 static gboolean on_item_leave(GtkWidget *item, GdkEventCrossing *e, gpointer d) {
 	if (hovered_item == item) hovered_item = NULL;
+	/*g_print("on_item_enter, hovered_item=%p\n",hovered_item);*/
 	return FALSE;
 }
 
@@ -1636,10 +1638,11 @@ static gboolean on_menu_key_press(GtkWidget *menu, GdkEventKey *event, gpointer
 	gchar *accel_path = NULL;
 	g_object_get(G_OBJECT(hovered_item), "accel-path", &accel_path, NULL);
 	if (!accel_path || !*accel_path) {
+		/*g_print("on_menu_key_press, accel_path=%s\n",accel_path);*/
 		g_free(accel_path);
 		return FALSE;
 	}
-	g_print("on_menu_key_press, change shortcut for accel_path=%s\n",accel_path);
+	DEBUG_MSG("on_menu_key_press, change shortcut for accel_path=%s\n",accel_path);
 	guint accel_key = event->keyval;
 	GdkModifierType accel_mods = event->state & gtk_accelerator_get_default_mod_mask();
 	
@@ -1649,9 +1652,10 @@ static gboolean on_menu_key_press(GtkWidget *menu, GdkEventKey *event, gpointer
 	
 	/* Rebind the action so it is immediately active */
 	GtkAction *action = gtk_activatable_get_related_action(GTK_ACTIVATABLE(hovered_item));
-	if (action)
+	/*g_print("action=%p for gtk_activatable_get_related_action()\n", action);*/
+	if (action){
 		gtk_action_set_accel_path(action, accel_path);
-	
+	}
 	/*gchar *label = gtk_accelerator_get_label(accel_key, accel_mods);
 	g_print("Nieuwe shortcut voor %s → %s\n", accel_path, label);
 	g_free(label);*/
@@ -1659,24 +1663,31 @@ static gboolean on_menu_key_press(GtkWidget *menu, GdkEventKey *event, gpointer
 	return TRUE;
 }
 
+void
+gtk3_enable_dynamic_menu_shortcuts_submenu(GtkWidget *submenu, gpointer data) {
+	/*g_print("gtk3_enable_dynamic_menu_shortcuts_submenu for submenu %p\n",submenu);*/
+	g_signal_connect(submenu, "key-press-event", G_CALLBACK(on_menu_key_press), NULL);
+}
+
 /* Connect hover/key handlers recursively */
 void 
 gtk3_enable_dynamic_menu_shortcuts_menuitem(GtkWidget *menuitem, gpointer data) {
 
+#ifdef DEBUG
 	GType type = G_OBJECT_TYPE(menuitem);
 	const char *type_name = g_type_name(type);
-	g_print ("menuitem_connect_signals, item %p has type %s\n", menuitem, type_name);
-	
+	g_print("menuitem_connect_signals, item %p has type %s\n", menuitem, type_name);
+#endif
 	if (!GTK_IS_MENU_ITEM(menuitem)) {
-		g_print("menuitem_connect_signals, geen menu_item, return\n"); 
+		g_print("menuitem_connect_signals, no menuitem, return\n"); 
 		return;
 	}
-	gchar *accel_path = NULL;
+	/*gchar *accel_path = NULL;
 	g_object_get(G_OBJECT(menuitem), "accel-path", &accel_path, NULL);
 	if (accel_path) {
 		g_print("menuitem_connect_signals, accel_path=%s\n",accel_path);
 		g_free(accel_path);
-	}
+	}*/
 	
 	g_signal_connect(menuitem, "enter-notify-event", G_CALLBACK(on_item_enter), NULL);
 	g_signal_connect(menuitem, "leave-notify-event", G_CALLBACK(on_item_leave), NULL);
@@ -1692,5 +1703,4 @@ gtk3_enable_dynamic_menu_shortcuts_menuitem(GtkWidget *menuitem, gpointer data)
 void gtk3_enable_dynamic_menu_shortcuts(GtkWidget *menubar, gpointer data) {
 	gtk_container_foreach(GTK_CONTAINER(menubar), gtk3_enable_dynamic_menu_shortcuts_menuitem, data);
 }
-#endif /* MAC_INTEGRATION */
 #endif /* GTK_CHECK_VERSION(3,0,0) */
