@@ -29,4 +29,12 @@
 #define _(String) dgettext(PACKAGE"_plugin_about", String)
 #endif
 
+typedef struct {
+	gint ignore_curversion_translation_message;
+	gint tested_curversion_translation_status;
+	gint ignore_new_user_dialog;
+} Tabout;
+
+extern Tabout about_v;
+
 #endif /* __ABOUT_H_  */
