@@ -61,15 +61,6 @@ htmlbar_doc_view_populate_popup(GtkTextView * textview, GtkMenu * menu, Tdocumen
 	}
 }
 
-static void
-htmlbar_doc_view_button_press(GtkWidget * widget, GdkEventButton * bevent, Tdocument * doc)
-{
-	DEBUG_MSG("htmlbar_doc_view_button_press, called\n");
-	if (bevent->button == 3) {
-		rpopup_bevent_in_html_code(doc);
-	}
-}
-
 static void
 htmlbar_sidepanel_initgui(Tbfwin *bfwin)
 {
@@ -139,6 +130,17 @@ htmlbar_init(void)
 	DEBUG_MSG("htmlbar_init, finished\n");
 }
 
+Thtmlbarsession *get_htmlbar_session(gpointer session) {
+	Thtmlbarsession *hbs = g_hash_table_lookup(htmlbar_v.lookup, session);
+	if (!hbs) {
+		hbs = g_new0(Thtmlbarsession, 1);
+		hbs->view_htmlbar = TRUE;
+		hbs->thumbnailwidth = 300;
+		g_hash_table_insert(htmlbar_v.lookup, session, hbs);
+	}
+	return hbs;
+}
+
 static void
 htmlbar_initgui(Tbfwin * bfwin)
 {
@@ -154,14 +156,7 @@ htmlbar_initgui(Tbfwin * bfwin)
 
 	g_hash_table_insert(htmlbar_v.lookup, bfwin, hbw);
 	DEBUG_MSG("htmlbar_initgui, adding hbw %p to hashtable %p with key %p\n", hbw, htmlbar_v.lookup, bfwin);
-	hbs = g_hash_table_lookup(htmlbar_v.lookup, bfwin->session);
-	if (!hbs) {
-		hbs = g_new0(Thtmlbarsession, 1);
-		hbs->view_htmlbar = TRUE;
-		DEBUG_MSG("htmlbar_initgui, adding hbs %p to hashtable %p with key %p\n", hbs, htmlbar_v.lookup,
-				  bfwin->session);
-		g_hash_table_insert(htmlbar_v.lookup, bfwin->session, hbs);
-	}
+	hbs = get_htmlbar_session(bfwin->session);
 	DEBUG_MSG("htmlbar_initgui, created hbw=%p and hbs=%p, will call htmlbar_load_ui\n",hbw,hbs);
 	htmlbar_load_ui(hbw);
 	htmlbar_menu_create(hbw);
@@ -176,16 +171,21 @@ htmlbar_initgui(Tbfwin * bfwin)
 }
 
 static void
-htmlbar_enforce_session(Tbfwin * bfwin)
+htmlbar_enforce_session(Tbfwin * bfwin, Tuimode uimode)
 {
 	Thtmlbarsession *hbs;
 	Thtmlbarwin *hbw;
 	DEBUG_MSG("htmlbar_enforce_session, started for session %p\n", bfwin->session);
-	hbs = g_hash_table_lookup(htmlbar_v.lookup, bfwin->session);
+	hbs = get_htmlbar_session(bfwin->session);
 	hbw = g_hash_table_lookup(htmlbar_v.lookup, bfwin);
 	if (!hbs || !hbw) {
 		return;
 	}
+	DEBUG_MSG("htmlbar_enforce_session, uimode=%d, view_htmlbar will be %d\n",uimode, (uimode == ui_everything || uimode == ui_webdevfull));
+	if (uimode != ui_undefined) {
+		/* change ui settings */
+		hbs->view_htmlbar = (uimode == ui_everything || uimode == ui_webdevfull);
+	}
 	if (!htmlbar_v.in_sidepanel) {
 		htmlbar_toolbar_show(hbw, hbs, hbs->view_htmlbar);
 		bfwin_set_menu_toggle_item(hbw->actiongroup, "ViewHTMLToolbar", hbs->view_htmlbar);
@@ -240,15 +240,7 @@ htmlbar_register_session_config(GHashTable * configlist, Tsessionvars * session)
 {
 	Thtmlbarsession *hbs;
 	DEBUG_MSG("htmlbar_register_session_config, started for %p\n", session);
-	hbs = g_hash_table_lookup(htmlbar_v.lookup, session);
-	if (!hbs) {
-		hbs = g_new0(Thtmlbarsession, 1);
-		hbs->view_htmlbar = TRUE;
-		hbs->thumbnailwidth = 300;
-		g_hash_table_insert(htmlbar_v.lookup, session, hbs);
-		DEBUG_MSG("htmlbar_register_session_config, adding hbs %p to hashtable %p with key %p\n", hbs,
-				  htmlbar_v.lookup, session);
-	}
+	hbs = get_htmlbar_session(session);
 	configlist = make_config_list_item(configlist, &hbs->view_htmlbar, 'i', "htmlbar_view:", 0);
 	configlist = make_config_list_item(configlist, &hbs->notebooktab, 'i', "htmlbar_notebooktab:", 0);
 	configlist = make_config_list_item(configlist, &hbs->thumbnailwidth, 'i', "htmlbar_thumbnailwidth:", 0);
