@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * snippets_gui.c - plugin for snippets sidebar
  *
- * Copyright (C) 2006-2012 Olivier Sessink
+ * Copyright (C) 2006-2026 Olivier Sessink
  * Copyright (C) 2011-2012 James Hayward
  *
  * This program is free software; you can redistribute it and/or modify
@@ -165,7 +165,21 @@ accelerator_cbdata_free(gpointer data)
 	g_slice_free(Taccelerator_cbdata, data);
 }
 
-static void
+static GClosure *connect_accelerator_real(Tsnippetswin * snw, xmlNodePtr cur) {
+	GClosure *closure;
+	Taccelerator_cbdata *hcbdata;
+	DEBUG_MSG("snippets.so:connect_accelerator_real() called\n");
+	hcbdata = g_slice_new(Taccelerator_cbdata);
+	hcbdata->snw = snw;
+	hcbdata->cur = cur;
+	closure =
+		g_cclosure_new(G_CALLBACK(snippets_accelerator_activated_lcb), hcbdata,
+					   (GClosureNotify) accelerator_cbdata_free);
+	g_object_watch_closure(G_OBJECT(snw->bfwin->main_window), closure);
+	return closure;
+}
+
+/*static void
 snippets_connect_accelerators_from_doc(Tsnippetswin * snw, xmlNodePtr cur, GtkAccelGroup * accel_group)
 {
 	DEBUG_MSG("snippets_connect_accelerators_from_doc\n");
@@ -175,26 +189,15 @@ snippets_connect_accelerators_from_doc(Tsnippetswin * snw, xmlNodePtr cur, GtkAc
 			snippets_connect_accelerators_from_doc(snw, cur, accel_group);
 		} else if ((xmlStrEqual(cur->name, (const xmlChar *) "leaf"))) {
 			xmlChar *accelerator;
-			/* if there are shortcut keys defined for this leaf we should register them */
+			/ * if there are shortcut keys defined for this leaf we should register them * /
 			accelerator = xmlGetProp(cur, (const xmlChar *) "accelerator");
 			if (accelerator) {
 				guint key;
 				GdkModifierType mod;
 				gtk_accelerator_parse((const gchar *) accelerator, &key, &mod);
 				if (key != 0 && (mod != 0 || (key >= GDK_F1 && key <= GDK_F12)) && gtk_accelerator_valid(key, mod)) {
-					GClosure *closure;
-					Taccelerator_cbdata *hcbdata;
-					hcbdata = g_slice_new(Taccelerator_cbdata);
-					hcbdata->snw = snw;
-					hcbdata->cur = cur;
-					DEBUG_MSG
-						("snippets_connect_accelerators_from_doc, connecting accelerator %s to data %p, and add to accel_group %p\n",
-						 accelerator, hcbdata, accel_group);
-					closure =
-						g_cclosure_new(G_CALLBACK(snippets_accelerator_activated_lcb), hcbdata,
-									   (GClosureNotify) accelerator_cbdata_free);
+					GClosure *closure = connect_accelerator_real(snw, cur);
 					gtk_accel_group_connect(accel_group, key, mod, GTK_ACCEL_VISIBLE, closure);
-					g_object_watch_closure(G_OBJECT(snw->bfwin->main_window), closure);
 				} else {
 					g_print("Invalid shortcut key %s found in snippets library\n", accelerator);
 				}
@@ -203,14 +206,46 @@ snippets_connect_accelerators_from_doc(Tsnippetswin * snw, xmlNodePtr cur, GtkAc
 		}
 		cur = cur->next;
 	}
+}*/
+
+static void
+snippets_fill_accelgroup_from_tree(Tsnippetswin * snw, GtkAccelGroup * accel_group, GtkTreeIter *parent) {
+	GtkTreeIter iter;
+	gboolean next;
+	DEBUG_MSG("snippets.so:snippets_fill_accelgroup_from_tree() called\n");
+	snippets_v.connecting_accelerators=TRUE;
+	next = gtk_tree_model_iter_children(GTK_TREE_MODEL(snippets_v.store),&iter,parent);
+	while(next) {
+		const gchar *accelpath;
+		xmlNodePtr node;
+		gtk_tree_model_get(GTK_TREE_MODEL(snippets_v.store),&iter,NODE_COLUMN, &node,ACCELPATH_COLUMN, &accelpath, -1);
+		if (node && accelpath) {
+			DEBUG_MSG("snippets_fill_accelgroup_from_tree, connect accelpath %s\n",accelpath);
+			/*guint key;
+			GdkModifierType mod;*/
+			GClosure *closure = connect_accelerator_real(snw, node);
+			gtk_accel_group_connect_by_path(accel_group,accelpath,closure);
+/*			gtk_accelerator_parse((const gchar *) accelerator, &key, &mod);
+			if (key!=0 || mod !=0) {
+				g_print("gtk_accel_map_add_entry %s for accelpath %s\n",accelerator, accelpath);
+				gtk_accel_map_add_entry(accelpath,key,mod);
+			}*/
+		}
+		
+		if (gtk_tree_model_iter_has_child(GTK_TREE_MODEL(snippets_v.store),&iter)) {
+			snippets_fill_accelgroup_from_tree(snw, accel_group, &iter);
+		}
+		next = gtk_tree_model_iter_next(GTK_TREE_MODEL(snippets_v.store),&iter);
+	}
+	snippets_v.connecting_accelerators=FALSE;
 }
 
 void
-snippets_rebuild_accelerators(void)
+snippets_rebuild_accelgroup(void)
 {
 	/* loop over all windows and rebuild the accelerators */
 	GList *tmplist;
-	DEBUG_MSG("snippets_rebuild_accelerators\n");
+	DEBUG_MSG("snipptets.so:snippets_rebuild_accelgroup() started\n");
 	for (tmplist = g_list_first(main_v->bfwinlist); tmplist; tmplist = tmplist->next) {
 		Tbfwin *bfwin = tmplist->data;
 		Tsnippetswin *snw = g_hash_table_lookup(snippets_v.lookup, bfwin);
@@ -223,12 +258,15 @@ snippets_rebuild_accelerators(void)
 		snw->accel_group = gtk_accel_group_new();
 		DEBUG_MSG("add new accel_group %p to bfwin %p\n",snw->accel_group,bfwin);
 		gtk_window_add_accel_group(GTK_WINDOW(bfwin->main_window), snw->accel_group);
-		if (snippets_v.doc) {
+		
+		snippets_fill_accelgroup_from_tree(snw, snw->accel_group, NULL);
+		
+		/*if (snippets_v.doc) {
 			xmlNodePtr cur = xmlDocGetRootElement(snippets_v.doc);
 			if (cur) {
 				snippets_connect_accelerators_from_doc(snw, cur, snw->accel_group);
 			}
-		}
+		}*/
 	}
 }
 
@@ -264,6 +302,7 @@ snippetsmenu_cb(gpointer user_data, gpointer data)
 void
 snippets_show_as_menu(Tsnippetswin * snw, gboolean enable)
 {
+	DEBUG_MSG("snippets_show_as_menu, enable=%d\n",enable);
 	if (enable) {
 		if (!snw->snippetsmenu) {
 			gint width = gdk_screen_get_width(gtk_window_get_screen(GTK_WINDOW(snw->bfwin->main_window)));
@@ -275,8 +314,8 @@ snippets_show_as_menu(Tsnippetswin * snw, gboolean enable)
 			gtk_container_add(GTK_CONTAINER(snw->bfwin->toolbarbox),snw->snippetsmenu);
 #endif
 			gtk_widget_show(snw->snippetsmenu);	/* show is required such that the size requests are accurate */
-			snippets_menu_set_model((SnippetsMenu *) snw->snippetsmenu, (GtkTreeModel *) snippets_v.store,
-									snippetsmenu_cb, snw, TITLE_COLUMN, NODE_COLUMN);
+			snippets_menu_set_model((SnippetsMenu *) snw->snippetsmenu, (GtkTreeModel *) snippets_v.store, snw->accel_group,
+									snippetsmenu_cb, snw, TITLE_COLUMN, NODE_COLUMN, ACCELPATH_COLUMN);
 		} else
 			gtk_widget_show(snw->snippetsmenu);
 	} else if (snw->snippetsmenu) {
@@ -549,7 +588,7 @@ snippetview_drag_data_received_lcb(GtkWidget * widget, GdkDragContext * context,
 					snippets_fill_tree_item_from_node(&newiter, srcnode);
 				}
 			}
-			g_idle_add(snippets_store_lcb, NULL);
+			trigger_snippets_store();
 			gtk_drag_finish(context, TRUE, TRUE, time);
 			return;
 		}
@@ -669,7 +708,7 @@ popup_menu_delete(GtkAction * action, gpointer user_data)
 		snw->lastclickednode = NULL;
 		gtk_tree_path_free(snw->lastclickedpath);
 		snw->lastclickedpath = NULL;
-		g_idle_add(snippets_store_lcb, NULL);
+		trigger_snippets_store();
 	}
 }
 
@@ -775,9 +814,19 @@ popup_menu_set_accelerator(GtkAction * action, gpointer user_data)
 			} else {
 				xmlSetProp(snw->lastclickednode, (const xmlChar *) "accelerator", (const xmlChar *) accel);
 			}
-			DEBUG_MSG("popup_menu_set_accelerator, call snippets_rebuild_accelerators()\n");
-			snippets_rebuild_accelerators();
-			g_idle_add(snippets_store_lcb, NULL);
+			GtkTreeIter iter;
+			if (gtk_tree_model_get_iter(GTK_TREE_MODEL(snippets_v.store), &iter, snw->lastclickedpath)) {
+				gchar *accelpath=NULL;
+				gtk_tree_model_get(GTK_TREE_MODEL(snippets_v.store),&iter,ACCELPATH_COLUMN, &accelpath, -1);
+				if (accelpath) {
+					set_accelerator_for_accelpath(accel, accelpath);
+					g_free(accelpath);
+				}
+			}
+			
+			/*DEBUG_MSG("popup_menu_set_accelerator, call snippets_rebuild_accelerators()\n");
+			snippets_rebuild_accelerators();*/
+			trigger_snippets_store();
 			g_free(accel);
 		}
 	}
@@ -877,12 +926,14 @@ snippets_create_gui(Tbfwin * bfwin)
 	snw->accel_group = gtk_accel_group_new();
 	gtk_window_add_accel_group(GTK_WINDOW(bfwin->main_window), snw->accel_group);
 	DEBUG_MSG("snippets_create_gui, connect accelerators to bfwin %p\n",bfwin);
-	if (snippets_v.doc) {
+	snippets_fill_accelgroup_from_tree(snw, snw->accel_group, NULL);
+	/*if (snippets_v.doc) {
 		xmlNodePtr cur = xmlDocGetRootElement(snippets_v.doc);
 		if (cur) {
 			snippets_connect_accelerators_from_doc(snw, cur, snw->accel_group);
 		}
-	}
+	}*/
+	
 
 	if (sns->show_as_menu) {
 		snippets_show_as_menu(snw, TRUE);
