@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * snippets_load.h - plugin for snippets sidebar
  *
- * Copyright (C) 2006 Olivier Sessink
+ * Copyright (C) 2006-2026 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,11 +25,11 @@ enum {
 	pixmap_type_insert,
 	pixmap_type_snr
 };
-
+void set_accelerator_for_accelpath(const gchar *accelerator, const gchar *accelpath);
 void snippets_fill_tree_item_from_node(GtkTreeIter *iter, xmlNodePtr node);
 
 void snippets_load(void);
-gboolean snippets_store_lcb(gpointer data);
+void trigger_snippets_store();
 void reload_tree_from_doc(void);
 void snippets_export_node(xmlNodePtr node, const gchar *filename);
 void snippets_import_node(xmlNodePtr branch, const gchar *filename);
