@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * snippets_wizard.c - plugin for snippets sidebar
  *
- * Copyright (C) 2006,2009-2011 Olivier Sessink
+ * Copyright (C) 2006,2009-2026 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -559,7 +559,7 @@ static gint snippets_test_pageBranch(Tsnipwiz *snwiz, gpointer data) {
 	
 	DEBUG_MSG("add branch with title %s\n",name);
 	g_free(name);
-	g_idle_add(snippets_store_lcb, NULL);
+	trigger_snippets_store();
 	return page_finished;
 }
 
@@ -735,7 +735,7 @@ static void snipwiz_dialog_response_lcb(GtkDialog *dialog, gint response, Tsnipw
 				if (snwiz->name) g_free(snwiz->name);
 				if (snwiz->description) g_free(snwiz->description);
 				g_free(snwiz);
-				g_idle_add(snippets_store_lcb, NULL);
+				trigger_snippets_store();
 				return;
 			break;
 		}
