@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * snippetsmenu.c - menubar that represents the treemodel as menu
  *
- * Copyright (C) 2009 Olivier Sessink
+ * Copyright (C) 2009-2026 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,6 +19,8 @@
 /*#define DEBUG*/
 #include <gtk/gtk.h>
 #include "../bluefish.h"
+#include "../gtk_easy.h"
+#include "snippets.h"
 #include "snippetsmenu.h"
 
 #if !GTK_CHECK_VERSION(2,22,0)
@@ -101,6 +103,12 @@ static void snippets_menu_row_inserted(GtkTreeModel * tree_model,
 				g_warning("indices should not be NULL!!!!!!!!!\n");
 			newitem = (GtkMenuItem *) gtk_menu_item_new_with_label("");
 			DEBUG_MSG("insert menu item %p at position %d\n",newitem, indices[0]);
+#if GTK_CHECK_VERSION(3,0,0)
+			if (main_v->props.dynamic_menu_shortcut) {
+				/*g_print("enable dynamic shortcut stuff for menuitem %p\n",newitem);*/
+				gtk3_enable_dynamic_menu_shortcuts_menuitem(GTK_WIDGET(newitem), NULL);
+			}
+#endif
 			gtk_menu_shell_insert((GtkMenuShell *)sm, (GtkWidget *)newitem, indices[0]);
 			gtk_widget_show((GtkWidget *)newitem);
 		}
@@ -113,6 +121,12 @@ static void snippets_menu_row_inserted(GtkTreeModel * tree_model,
 			if (!mshell) {
 				GtkMenuItem *tearoff;
 				mshell = (GtkMenuShell *)gtk_menu_new();
+				gtk_menu_set_accel_group(GTK_MENU(mshell), sm->accel_group);
+#if GTK_CHECK_VERSION(3,0,0)
+				if (main_v->props.dynamic_menu_shortcut) {
+					gtk3_enable_dynamic_menu_shortcuts_submenu(GTK_WIDGET(mshell), NULL);
+				}
+#endif
 				DEBUG_MSG("append mshell %p to item %p\n",mshell, item);
 				gtk_menu_item_set_submenu(item, (GtkWidget *)mshell);
 				tearoff = (GtkMenuItem *) gtk_tearoff_menu_item_new();
@@ -121,6 +135,12 @@ static void snippets_menu_row_inserted(GtkTreeModel * tree_model,
 			}
 			DEBUG_MSG("row inserted, insert in mshell=%p at position %d\n",mshell, indices[depth-1]+1);
 			newitem = (GtkMenuItem *) gtk_menu_item_new_with_label("");
+#if GTK_CHECK_VERSION(3,0,0)
+			if (main_v->props.dynamic_menu_shortcut) {
+				/*g_print("enable dynamic shortcut stuff for menuitem %p\n",newitem);*/
+				gtk3_enable_dynamic_menu_shortcuts_menuitem(GTK_WIDGET(newitem), NULL);
+			}
+#endif
 			/* add 1 to the index number for the tearoff item */
 			gtk_menu_shell_insert((GtkMenuShell *)mshell, (GtkWidget *)newitem, indices[depth-1]+1);
 			gtk_widget_show((GtkWidget *)newitem);
@@ -181,14 +201,21 @@ static void snippets_menu_row_changed(GtkTreeModel * tree_model,
 	mitem = menuitem_from_path(sm, path);
 	if (mitem) {
 		gchar *name=NULL;
+		gchar *accelpath=NULL;
 		gpointer pointer;
 		Tsmdata *smdata;
-		gtk_tree_model_get(tree_model, iter, sm->name_column, &name, sm->data_column, &pointer, -1);
+		gtk_tree_model_get(tree_model, iter, sm->name_column, &name, sm->data_column, &pointer, sm->accel_column, &accelpath, -1);
 		if (gtk_bin_get_child(GTK_BIN(mitem))) {
 			g_signal_handlers_disconnect_matched(mitem, G_SIGNAL_MATCH_FUNC, 0, 0, NULL, menuitem_activate, NULL);
 			gtk_label_set_text(GTK_LABEL(gtk_bin_get_child(GTK_BIN(mitem))),name);
 			g_free(name);
 		}
+		if (accelpath) {
+			DEBUG_MSG("snippets_menu_row_changed() set accelpath %s for menuitem %p\n",accelpath,mitem);
+			gtk_menu_item_set_accel_path(mitem, accelpath);
+			/*gtk_widget_set_accel_path((GtkWidget *)mitem, accelpath, sm->accel_group);*/
+			g_free(accelpath);
+		}
 		smdata = g_object_get_data(G_OBJECT(mitem),"smdata");
 		if (!smdata) {
 			smdata = g_slice_new(Tsmdata);
@@ -209,19 +236,22 @@ gboolean snippets_menu_set_model_foreach(GtkTreeModel *model,GtkTreePath *path,G
 	snippets_menu_row_changed(model,path, iter, data);
 	return FALSE;
 }
-void snippets_menu_set_model(SnippetsMenu * sm, GtkTreeModel * model, SnippetMenuCallback callback, gpointer user_data, gint name_column, gint data_column)
+void snippets_menu_set_model(SnippetsMenu * sm, GtkTreeModel * model,GtkAccelGroup *accel_group, SnippetMenuCallback callback, gpointer user_data, gint name_column, gint data_column, gint accel_column)
 {
 	sm->name_column = name_column;
 	sm->data_column = data_column;
+	sm->accel_column = accel_column;
 	sm->callback = callback;
 	sm->user_data = user_data;
+	sm->accel_group = accel_group;
 	g_signal_connect(model, "row-changed", G_CALLBACK(snippets_menu_row_changed), sm);
 	g_signal_connect(model, "row-deleted", G_CALLBACK(snippets_menu_row_deleted), sm);
 	/*g_signal_connect(model, "row-has-child-toggled", G_CALLBACK(snippets_menu_row_has_child_toggled), sm);
 	g_signal_connect(model, "rows-reordered", G_CALLBACK(snippets_menu_rows_reordered), sm);*/
 	g_signal_connect(model, "row-inserted", G_CALLBACK(snippets_menu_row_inserted), sm);
-	
+	snippets_v.connecting_accelerators=TRUE;
 	gtk_tree_model_foreach(model, snippets_menu_set_model_foreach, sm);
+	snippets_v.connecting_accelerators=FALSE;
 }
 
 
