@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * project.c - project functionality
  *
- * Copyright (C) 2003-2023 Olivier Sessink
+ * Copyright (C) 2003-2026 Olivier Sessink
  * Copyright (C) 2011 James Hayward
  *
  * This program is free software; you can redistribute it and/or modify
@@ -160,7 +160,7 @@ set_project_menu_actions(Tbfwin * bfwin, gboolean win_has_project)
 
 
 static void
-setup_bfwin_for_project(Tbfwin * bfwin, Tdocument *active_doc)
+setup_bfwin_for_project(Tbfwin * bfwin, Tdocument *active_doc, Tuimode uimode)
 {
 	DEBUG_MSG("setup_bfwin_for_project, bfwin=%p, bfwin->project=%p, bfwin->session=%p\n, current_document=%p, active_doc=%p\n", bfwin,
 			  bfwin->project, bfwin->session, bfwin->current_document, active_doc);
@@ -176,7 +176,6 @@ setup_bfwin_for_project(Tbfwin * bfwin, Tdocument *active_doc)
 #ifdef HAVE_LIBENCHANT
 	reload_spell_dictionary(bfwin);
 #endif
-	bfwin_apply_session(bfwin, active_doc);
 	set_project_menu_actions(bfwin, TRUE);
 #ifdef MAC_INTEGRATION
 /*	ige_mac_menu_sync(GTK_MENU_SHELL(BFWIN(doc->bfwin)->menubar));*/
@@ -197,7 +196,7 @@ setup_bfwin_for_nonproject(Tbfwin * bfwin)
 	if (bfwin->current_document)
 		bfwin_set_title(bfwin, bfwin->current_document, 0);
 
-	bfwin_apply_session(bfwin, bfwin->current_document);
+	bfwin_apply_session(bfwin, bfwin->current_document, ui_undefined);
 	set_project_menu_actions(bfwin, FALSE);
 #ifdef MAC_INTEGRATION
 /*	ige_mac_menu_sync(GTK_MENU_SHELL(BFWIN(bfwin)->menubar));*/
@@ -221,7 +220,13 @@ project_destroy(Tproject * project)
 	g_free(project);
 }
 
-/* bfwin is allowed to be NULL for an empty project */
+/*  create_new_project() creates a new project
+if the current window has a single empty unnamed document it is re-used for this project
+in other cases (bfwibn will be NULL) there will be a new window opened for this project after it is created
+
+*** bfwin is allowed to be NULL for an empty project ***
+ 
+*/
 static Tproject *
 create_new_project(Tbfwin * bfwin)
 {
@@ -229,9 +234,9 @@ create_new_project(Tbfwin * bfwin)
 	prj = g_new0(Tproject, 1);
 	DEBUG_MSG("create_new_project, project=%p, bfwin=%p\n", prj, bfwin);
 	prj->name = g_strdup(_("New project"));
-	prj->bmarkdata = bookmark_data_new();
 	prj->session = g_new0(Tsessionvars, 1);
 	project_setup_initial_session(prj->session, FALSE);
+	prj->bmarkdata = bookmark_data_new();
 	/*Project MUST have recent_dirs*/
 	if (bfwin && bfwin->current_document && bfwin->current_document->uri) {/* We have document with uri */
 		GFile *dir_uri = g_file_get_parent(bfwin->current_document->uri);
@@ -277,7 +282,7 @@ create_new_project(Tbfwin * bfwin)
 		}
 		bfwin->project = prj;
 		bfwin->session = prj->session;
-		setup_bfwin_for_project(bfwin, NULL);
+		setup_bfwin_for_project(bfwin, NULL, ui_undefined);
 	}
 	return prj;
 }
@@ -428,6 +433,9 @@ project_open_from_file(Tbfwin * bfwin, GFile * fromuri)
 		/* destroy the current empty document, it should use settings from the new session */
 		if (prwin->current_document)
 			doc_destroy(prwin->current_document, TRUE);
+		bfwin->current_document=NULL;
+		bfwin->notebook_page=-1;
+		bfwin->notebook2_page=-1;
 	} else {
 		/* we will open a new Bluefish window for this project */
 		DEBUG_MSG("project_open_from_file, we need a new window\n");
@@ -474,13 +482,14 @@ project_open_from_file(Tbfwin * bfwin, GFile * fromuri)
 		/* Now switch to tab that holds last active document from previous session */
 		bfwin_notebook_unblock_signals(prwin);  /* Unblock signals, doc will be activated in setup_bfwin_for_project*/
 		bfwin_switch_to_document_by_index(prwin, doc_index);
-		bfwin_notebook_changed(prwin, doc_index);
+		bfwin_notebook_changed(prwin, prwin->notebook, doc_index);
 	} else {
 		doc_new(prwin, FALSE);
 	}
 	Tdocument * active_doc = documentlist_return_document_from_index(prwin->documentlist, doc_index);
 	DEBUG_MSG("project_open_from_file, new window with files ready at prwin=%p\n", prwin);
-	setup_bfwin_for_project(prwin, active_doc);
+	setup_bfwin_for_project(prwin, active_doc, ui_undefined);
+	bfwin_apply_session(bfwin, active_doc, ui_undefined);
 	DEBUG_MSG("project_open_from_file, done\n");
 	return 1;
 }
@@ -489,7 +498,7 @@ static void
 project_open_response_lcb(GtkDialog * dialog, gint response, Tbfwin * bfwin)
 {
 	if (response == GTK_RESPONSE_ACCEPT) {
-		GFile *file, *dir;
+		GFile *file;
 		file = gtk_file_chooser_get_file(GTK_FILE_CHOOSER(dialog));
 		project_open_from_file(bfwin, file);
 		set_last_project_dir_from_file_uri(file);
@@ -532,10 +541,11 @@ project_final_close(Tbfwin * bfwin, gboolean close_win)
 {
 	if (!bfwin->project)
 		return TRUE;
-
+	DEBUG_MSG("project_final_close, call project_destroy(%p)\n",bfwin->project);
 	project_destroy(bfwin->project);
 	/* we should only set the window for nonproject if the window will keep alive */
 	if (!close_win) {
+		DEBUG_MSG("project_final_close, call setup_bfwin_for_nonproject(%p)\n",bfwin);
 		setup_bfwin_for_nonproject(bfwin);
 	}
 	return TRUE;
@@ -582,6 +592,7 @@ typedef struct {
 	Tbfwin *bfwin;
 	Tproject *project;
 	Tsessionprefs sprefs;
+	Tdefault_ui du;
 	GtkWidget *entries[projecteditor_entries_num];
 	gboolean destroy_project_on_close;
 } Tprojecteditor;
@@ -620,11 +631,12 @@ project_edit_ok_clicked_lcb(GtkWidget * widget, Tprojecteditor * pred)
 	gtk_widget_hide(pred->win);
 	DEBUG_MSG("project_edit_ok_clicked_lcb, Tproject at %p, bfwin at %p\n", prj, pred->bfwin);
 	string_apply(&prj->name, pred->entries[name]);
+	
 	sessionprefs_apply(&pred->sprefs, pred->project->session);
 	if (pred->bfwin == NULL) {
 		pred->bfwin = bfwin_window_new_with_project(pred->project, TRUE);
 		pred->bfwin->session = pred->project->session;
-		setup_bfwin_for_project(pred->bfwin, NULL);
+		setup_bfwin_for_project(pred->bfwin, NULL, pred->du.uimode);
 	} else {
 #ifdef MAC_INTEGRATION
 	if (main_v->osx_status == 2){
@@ -635,7 +647,7 @@ project_edit_ok_clicked_lcb(GtkWidget * widget, Tprojecteditor * pred)
 #endif
 		bfwin_set_title(pred->bfwin, pred->bfwin->current_document, 0);
 	}
-/* set_project_menu_widgets(pred->bfwin, TRUE);*/
+	bfwin_apply_session(pred->bfwin, pred->bfwin->current_document, pred->du.uimode);
 	project_save(pred->bfwin, FALSE);
 	gtk_widget_destroy(pred->win);
 }
@@ -646,7 +658,7 @@ if NULL is passed as bfwin, a new window will be created for this project once O
 void
 project_edit(Tbfwin * bfwin)
 {
-	GtkWidget *vbox, *but, *hbox, *label, *table, *parent_window=NULL;
+	GtkWidget *notebook, *vbox, *but, *hbox, *label, *table, *parent_window=NULL;
 	gchar *wintitle = NULL;
 	Tprojecteditor *pred;
 
@@ -707,16 +719,20 @@ project_edit(Tbfwin * bfwin)
 	pred->entries[name] = dialog_entry_in_table(pred->project->name, table, 1, 2, 0, 1);
 	gtk_widget_set_size_request(GTK_WIDGET(pred->entries[name]), 250, -1);
 	dialog_mnemonic_label_in_table(_("Project _Name:"), pred->entries[name], table, 0, 1, 0, 1);
-
-/*	pred->entries[template] = entry_with_text(pred->project->template, 255);
-	bf_mnemonic_label_tad_with_alignment(_("_Template:"), pred->entries[template], 1, 0.5, table, 0, 1, 3, 4);
-	but = file_but_new(pred->entries[template], 1, NULL);
-	gtk_table_attach_defaults(GTK_TABLE(table), pred->entries[template], 2, 3, 3, 4);
-	gtk_table_attach_defaults(GTK_TABLE(table), but, 3, 4, 3, 4);*/
-
+	
 	sessionprefs(_("<b>Default settings for this project</b>"), &pred->sprefs, pred->project->session);
-	gtk_box_pack_start(GTK_BOX(vbox), pred->sprefs.frame, FALSE, FALSE, 6);
+	if (pred->destroy_project_on_close) {
+		notebook = gtk_notebook_new();
+		gtk_box_pack_start(GTK_BOX(vbox), notebook, FALSE, FALSE, 6);
 
+		default_ui_settings(_("<b>User interface for this project</b>"), &pred->du);
+		gtk_notebook_append_page(GTK_NOTEBOOK(notebook),pred->du.grid, gtk_label_new(_("User interface layout")));
+		gtk_container_set_border_width(GTK_CONTAINER(pred->du.grid), 10);	
+		gtk_notebook_append_page(GTK_NOTEBOOK(notebook),pred->sprefs.frame, gtk_label_new(_("Initial document settings")));
+	} else { 
+		gtk_box_pack_start(GTK_BOX(vbox), pred->sprefs.frame, FALSE, FALSE, 6);
+	}
+	
 #if GTK_CHECK_VERSION(3,0,0)
 	hbox = gtk_button_box_new(GTK_ORIENTATION_HORIZONTAL);
 #else
@@ -734,6 +750,7 @@ project_edit(Tbfwin * bfwin)
 	}
 	gtk_box_pack_start(GTK_BOX(hbox), but, FALSE, FALSE, 0);
 	gtk_widget_grab_default(but);
+	gtk_window_set_default_size (GTK_WINDOW(pred->win),500,-1);
 	gtk_widget_show_all(pred->win);
 }
 
