@@ -17,10 +17,10 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#define CURCONFIG "rcfile-2.0"
+/*#define CURCONFIG "rcfile-2.0"
 #define OLDCONFIG "rcfile_v2"
 #define CURSESSION "session-2.0"
-#define OLDSESSION "session"
+#define OLDSESSION "session"*/
 
 /*****
  * THE WORKING OF THE CONFIG FILES AND STRUCTURES IN BLUEFISH
@@ -83,6 +83,7 @@
 #include "document.h"			/* DOCUMENT_BACKUP_ABORT_ASK */
 #include "plugins.h"
 #include "stringlist.h"
+#include "paths_state_config.h"
 
 
 typedef struct {
@@ -445,6 +446,7 @@ props_init_main(GHashTable * config_rc)
 	init_prop_integer(&config_rc, &main_v->props.hide_bars_on_fullscreen, "hide_bars_on_fullscreen:", 1, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.cursor_size, "cursor_size:", 4, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.highlight_cursor, "highlight_cursor:", 0, TRUE);
+	init_prop_integer(&config_rc, &main_v->props.dynamic_menu_shortcut, "dynamic_menu_shortcut:", 0, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.save_accelmap, "save_accelmap:", 1, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.max_recent_files, "max_recent_files:", 15, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.max_dir_history, "max_dir_history:", 10, TRUE);
@@ -547,7 +549,7 @@ props_init_main(GHashTable * config_rc)
 #endif
 	return config_rc;
 }
-
+/*
 static gboolean
 merge_config_files(GFile * oldrc, GFile * oldsession, GFile * newrc, GFile * newsession)
 {
@@ -607,10 +609,10 @@ migrate_config_files(GHashTable * main_configlist, GFile * newrc)
 	oldsession = user_bfdir(OLDSESSION);
 	newsession = user_bfdir(CURSESSION);
 	if (merge_config_files(oldrc, oldsession, newrc, newsession)) {
-	/* add some entries to main_configlist. because main_configlist is used for
-	   saving as well this means we'll save these entries too */
+	/ * add some entries to main_configlist. because main_configlist is used for
+	   saving as well this means we'll save these entries too * /
 		if (parse_config_file(main_configlist, newrc)) {
-			/* are there any entries that we want to convert ?? */
+			/ * are there any entries that we want to convert ?? * /
 			save_config_file(main_configlist, newrc);
 			return TRUE;
 		}
@@ -618,7 +620,7 @@ migrate_config_files(GHashTable * main_configlist, GFile * newrc)
 	}
 	return FALSE;
 }
-
+*/
 
 static gchar *
 convert_old_placeholders(gchar *orig, gchar old1, gchar new1, gchar old2, gchar new2)
@@ -874,10 +876,11 @@ GList *update_commands(GList *current, gboolean overwrite)
 {
 	GList *defaults=NULL, *retlist;
 #ifdef WIN32
-	defaults = add_browser_if_exists(defaults, _("Windows 7 Firefox 32-bit"), "C:\\Program Files (x86)\\Mozilla Firefox\\firefox.exe", "1");
-	defaults = add_browser_if_exists(defaults, _("Windows 7 Firefox"), "C:\\Program Files\\Mozilla Firefox\\firefox.exe", "1");
-	defaults = add_browser_if_exists(defaults, _("Windows 7 Internet Explorer 32-bit"), "C:\\Program Files (x86)\\Internet Explorer\\iexplore.exe", "0");
-	defaults = add_browser_if_exists(defaults, _("Windows 7 Internet Explorer 64-bit"), "C:\\Program Files\\Internet Explorer\\iexplore.exe", "0");
+	defaults = add_browser_if_exists(defaults, _("Firefox (x86)"), "C:\\Program Files (x86)\\Mozilla Firefox\\firefox.exe", "1");
+	defaults = add_browser_if_exists(defaults, _("Firefox"), "C:\\Program Files\\Mozilla Firefox\\firefox.exe", "1");
+	defaults = add_browser_if_exists(defaults, _("Internet Explorer (x86)"), "C:\\Program Files (x86)\\Internet Explorer\\iexplore.exe", "0");
+	defaults = add_browser_if_exists(defaults, _("Internet Explorer"), "C:\\Program Files\\Internet Explorer\\iexplore.exe", "0");
+	defaults = add_browser_if_exists(defaults, _("Edge (x86)"), "C:\\Program Files (x86)\\Microsoft\\Edge\\Application\\msedge.exe", "0");
 #elif PLATFORM_DARWIN
 	defaults =
 		g_list_prepend(defaults,
@@ -931,6 +934,42 @@ GList *update_commands(GList *current, gboolean overwrite)
 	free_arraylist(defaults);
 	return retlist;
 }
+/*
+void
+rcfile_check_directory(void)
+{
+	GFile *file, *rcpath, *rcsubpath;
+	GFileInfo *finfo;
+	GError *gerror = NULL;
+
+	file = g_file_new_for_path(g_get_home_dir());
+	rcpath = g_file_get_child(file, ".bluefish");
+
+	DEBUG_PATH("test if rcfile directory %s exists\n", rcpath);
+	finfo = g_file_query_info(rcpath, G_FILE_ATTRIBUTE_STANDARD_TYPE, G_FILE_QUERY_INFO_NONE, NULL, &gerror);
+	if (gerror && gerror->code == G_IO_ERROR_NOT_FOUND) {
+		g_clear_error(&gerror);
+
+		g_file_make_directory(rcpath, NULL, &gerror);
+		if (gerror) {
+			g_warning("Error creating directory: .bluefish\n");
+			g_warning("%s", gerror->message);
+			g_error_free(gerror);
+		}
+		rcsubpath = g_file_get_child(rcpath, "templates");
+		g_file_make_directory(rcsubpath, NULL, NULL);
+		g_object_unref(rcsubpath);
+		rcsubpath = g_file_get_child(rcpath, "autosave");
+		g_file_make_directory(rcsubpath, NULL, NULL);
+		g_object_unref(rcsubpath);
+	}
+
+	g_object_unref(rcpath);
+	g_object_unref(file);
+	if (finfo)
+		g_object_unref(finfo);
+}
+*/
 
 void
 rcfile_parse_main(void)
@@ -946,15 +985,11 @@ rcfile_parse_main(void)
 	main_configlist =
 		props_init_main(g_hash_table_new_full(g_str_hash, g_str_equal, NULL, free_config_list_item));
 
-	file = user_bfdir(CURCONFIG);
+	file = paths_config_gfile();
 	if (!parse_config_file(main_configlist, file)) {
-		g_warning("no configfile %s, try to convert config files from older versions\n", CURCONFIG);
-		/* probably there is no configfile. try to migrate the configfile from a previous
-		   version */
-		if (!migrate_config_files(main_configlist, file)) {
-			g_free(main_v->props.config_version);
-			main_v->props.config_version = NULL; 	/* We do not have any config file */	
-		}
+		g_warning("configfile failed to load, check directories and try to convert config files from older versions\n");
+		/* probably there is no configfile. try to migrate the configfile from a previous version */
+		paths_create_dirs_and_migrate();
 	}
 	g_object_unref(file);
 	if (main_v->props.encoding_search_Nbytes < 1000)
@@ -978,34 +1013,6 @@ rcfile_parse_main(void)
 		main_v->props.external_outputbox = update_outputbox(main_v->props.external_outputbox, FALSE);
 	if (!main_v->props.external_bookmarkcommands)
 		main_v->props.external_bookmarkcommands = update_bookmarkcommands(main_v->props.external_bookmarkcommands, FALSE);	
-	
-#ifdef OLDTEMPLATES
-	if (main_v->props.templates == NULL) {
-		main_v->props.templates =
-			g_list_append(main_v->props.templates,
-						  array_from_arglist(_("C Header"), PKGDATADIR "/templates/C_header", NULL));
-		main_v->props.templates =
-			g_list_append(main_v->props.templates,
-						  array_from_arglist(_("C Header GPL"), PKGDATADIR "/templates/C_header_GPL", NULL));
-		main_v->props.templates =
-			g_list_append(main_v->props.templates,
-						  array_from_arglist(_("XHTML 1.0"), PKGDATADIR "/templates/XHTML_1.0", NULL));
-		main_v->props.templates =
-			g_list_append(main_v->props.templates,
-						  array_from_arglist(_("HTML 5"), PKGDATADIR "/templates/HTML_5", NULL));
-		main_v->props.templates =
-			g_list_append(main_v->props.templates,
-						  array_from_arglist(_("PHP"), PKGDATADIR "/templates/PHP", NULL));
-		main_v->props.templates =
-			g_list_append(main_v->props.templates,
-						  array_from_arglist(_("LaTex Book"), PKGDATADIR "/templates/Latex_Book", NULL));
-		main_v->props.templates =
-			g_list_append(main_v->props.templates,
-						  array_from_arglist(_("LaTex Presentation"), PKGDATADIR "/templates/Latex_Presentation", NULL));
-		/* TODO: list the templates in the directory */
-	}
-#endif
-
 #ifdef WIN32
 	if (main_v->props.file_association == NULL) {
 /*
@@ -1120,7 +1127,7 @@ gint
 rcfile_save_main(void)
 {
 	gint ret;
-	GFile *filename = user_bfdir(CURCONFIG);
+	GFile *filename = paths_config_gfile();
 
 	g_free(main_v->props.config_version);
 	main_v->props.config_version = g_strdup(VERSION);
@@ -1130,57 +1137,17 @@ rcfile_save_main(void)
 	return ret;
 }
 
-void
-rcfile_check_directory(void)
-{
-	GFile *file, *rcpath, *rcsubpath;
-	GFileInfo *finfo;
-	GError *gerror = NULL;
-
-	file = g_file_new_for_path(g_get_home_dir());
-	rcpath = g_file_get_child(file, ".bluefish");
-
-	DEBUG_PATH("test if rcfile directory %s exists\n", rcpath);
-	finfo = g_file_query_info(rcpath, G_FILE_ATTRIBUTE_STANDARD_TYPE, G_FILE_QUERY_INFO_NONE, NULL, &gerror);
-	if (gerror && gerror->code == G_IO_ERROR_NOT_FOUND) {
-		g_clear_error(&gerror);
-
-		g_file_make_directory(rcpath, NULL, &gerror);
-		if (gerror) {
-			g_warning("Error creating directory: .bluefish\n");
-			g_warning("%s", gerror->message);
-			g_error_free(gerror);
-		}
-		rcsubpath = g_file_get_child(rcpath, "templates");
-		g_file_make_directory(rcsubpath, NULL, NULL);
-		g_object_unref(rcsubpath);
-		rcsubpath = g_file_get_child(rcpath, "autosave");
-		g_file_make_directory(rcsubpath, NULL, NULL);
-		g_object_unref(rcsubpath);
-	}
-
-	g_object_unref(rcpath);
-	g_object_unref(file);
-	if (finfo)
-		g_object_unref(finfo);
-}
-
-static gchar *
-get_user_accelmap(void)
-{
-	return g_strconcat(g_get_home_dir(), "/." PACKAGE "/menudump_2", NULL);
-}
-
 void
 rcfile_save_accelerators(void)
 {
 #ifndef MAC_INTEGRATION
-	gchar *shortcutfilename = get_user_accelmap();
+	gchar *shortcutfilename = paths_user_accelmap();
 	gtk_accel_map_save(shortcutfilename);
 	g_free(shortcutfilename);
 #endif
 }
-
+/* called from rcfile to load the user accelmap (FALSE), and from the preferences 
+to reset to the defaults (TRUE)*/
 void
 rcfile_load_accelerators(gboolean defaultmap)
 {
@@ -1189,7 +1156,7 @@ rcfile_load_accelerators(gboolean defaultmap)
 	if (defaultmap) {
 		shortcutfilename = g_strdup(PKGDATADIR"/default_accelmap");
 	} else {
-		shortcutfilename = get_user_accelmap();
+		shortcutfilename = paths_user_accelmap();
 	}
 	gtk_accel_map_load(shortcutfilename);
 	g_free(shortcutfilename);
@@ -1500,7 +1467,7 @@ rcfile_save_global_session(void)
 	gboolean retval;
 	GFile *filename;
 	GHashTable *configlist;
-	filename = user_bfdir(CURSESSION);
+	filename = paths_session_gfile();
 	configlist = return_globalsession_configlist(FALSE);
 	configlist = return_session_configlist(configlist, main_v->session);
 	DEBUG_MSG("rcfile_save_global_session, saving global session to %s\n", g_file_get_parse_name(filename));
@@ -1544,7 +1511,7 @@ rcfile_parse_global_session(void)
 	GFile *file;
 	GHashTable *configlist = return_globalsession_configlist(TRUE);
 	configlist = return_session_configlist(configlist, main_v->session);
-	file = user_bfdir(CURSESSION);
+	file = paths_session_gfile();
 	retval = parse_config_file(configlist, file);
 	free_configlist(configlist);
 	g_object_unref(file);
@@ -1642,7 +1609,7 @@ void
 load_templates(void)
 {
 	GList *t1, *t2;
-	GFile *uri = user_bfdir("templates/");
+	GFile *uri = paths_templatesdir_gfile();
 	t1 = load_templates_from_dir(uri, TRUE);
 	g_object_unref(uri);
 	uri = g_file_new_for_path(PKGDATADIR"/templates/");
