@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * undo_redo.h - undo structures and function declarations
  *
- * Copyright (C) 1998-2011 Olivier Sessink
+ * Copyright (C) 1998-2026 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@
 
 
 guint new_unre_action_id(void);
+/* adds the string text to the list of undo actions at provided positions with type UndoInsert or UndoDelete. a new copy of the string is created. */
 void doc_unre_add(Tdocument * doc, const gchar *text, guint32 start, guint32 end, undo_op_t op);
 void doc_unre_new_group_action_id(Tdocument * doc, guint32 action_id);
 void doc_unre_new_group(Tdocument * doc);
@@ -29,8 +30,9 @@ void doc_unre_init(Tdocument * doc);
 void doc_unre_destroy(Tdocument * doc);
 void doc_unre_clear_all(Tdocument * doc);
 void doc_unre_clear_not_modified(Tdocument * doc);
-gint doc_undo_op_compare(Tdocument * doc, undo_op_t testfor, gint position);
+/*gint doc_undo_op_compare(Tdocument * doc, undo_op_t testfor, gint position);*/
 gboolean doc_unre_test_last_entry(Tdocument * doc, undo_op_t testfor, guint32 start, guint32 end);
+void undo_doc(Tdocument *doc, gboolean noscroll);
 void redo(Tbfwin * bfwin);
 void undo(Tbfwin * bfwin);
 void redo_all(Tbfwin * bfwin);
